#
# cmake PackageConfig file for BIAS
#
# NOTES
# Typical usage in user client project:
#   FIND_PACKAGE(BIAS)
#   IF (BIAS_FOUND)
#     INCLUDE(${BIAS_USE_FILE})
#   ENDIF (BIAS_FOUND)
#
# or:
#
#   FIND_PACKAGE(BIAS REQUIRED)
#   INCLUDE(${BIAS_USE_FILE})
#
# AUTHOR
# Jan Woetzel 06/2004-2006

# where to find the USE file to be used by user project
# include this file in client projects
SET(BIAS_USE_FILE "/usr/local/include/BIAS/UseBIAS.cmake")

# where are our (installed) headers
# and third party headers we depend on
SET(BIAS_INCLUDE_DIR
  "/usr/local"
  "/usr/local/include/BIAS"
  "/usr/include;/usr/include;/usr/include;/usr/include;/usr/include;/opt/net/gcc41/OpenCV-1.0.0/include/opencv;/opt/net/gcc41/OpenCV-1.0.0/include/opencv;/opt/net/gcc41/OpenCV-1.0.0/include/opencv;/opt/net/gcc41/OpenCV-1.0.0/include/opencv;/usr/include;/usr/lib/wx/include/gtk2-unicode-release-2.8;/usr/include/wx-2.8;-pthread" )

# where are our lib(s)
# third party libs are found via their absolute dependency paths
# install or use build tree?
SET(BIAS_LINK_DIRECTORIES
  "/usr/local/lib/shared"
  ""
  "/usr/local")

SET(BIAS_LINK_DIRECTORIES ${BIAS_LINK_DIRECTORIES}
  "/usr/lib;/usr/lib;/usr/lib;/usr/lib;/opt/net/gcc41/OpenCV-1.0.0/lib;/usr/lib;/usr/lib" )


# where to find our build settings and dependencies
SET(BIAS_BUILD_SETTINGS_FILE "/usr/local/BIASBuildSettings.cmake")

SET(BIAS_BUILD_SHARED_LIBS "ON")
INCLUDE("/usr/local/BIASLibraryDepends.cmake")

# export source dir for doxygen
SET(BIAS_SOURCE_DIR "/usr/local/include/BIAS")

# set the (one) library child projects can use 
# (if they dont want to solve dependencies theirselves)
SET(BIAS_LIBRARIES BIASCommon;BIASDebug;BIASMathBase;BIASGeometryBase;BIASImageBase;BIASMathAlgo;BIASGeometry;BIASStateEstimator;BIASVideoSource;BIASGui;BIASFilter;BIASImage;BIASFeatureDetector;BIASUtils;BIASMatcher2D)


# additional build settings
SET(BIAS_LIBRARY_OUTPUT_PATH    "/usr/local/lib/shared")
SET(BIAS_EXECUTABLE_OUTPUT_PATH "")
SET(BIAS_CMAKE_SYSTEM_PROCESSOR "i686")
SET(BIAS_CMAKE_SYSTEM_NAME      "Linux")
SET(BIAS_CMAKE_BUILD_TOOL       "/usr/bin/gmake")
SET(BIAS_BUILD_TYPE             "Debug")
IF(WIN32)
  SET(BIAS_CONFIGURATION_TYPES  "")
ENDIF(WIN32)


#
# remember BIAS' third party libs configuration:
#

SET(BIAS_HAVE_BLAS           "ON")
SET(BIAS_DAIMLERCHRYSLER     "")
SET(BIAS_HAVE_1394CAMERACMU  "")
SET(BIAS_HAVE_DC1394         "")
SET(BIAS_HAVE_DSHOW          "")
SET(BIAS_HAVE_DEVIL          "")
SET(BIAS_HAVE_DV             "")
SET(BIAS_HAVE_FFTW3          "")
SET(BIAS_HAVE_GLEW           "ON")
SET(BIAS_HAVE_GLUT           "ON")
SET(BIAS_HAVE_GSL            "ON")
SET(BIAS_HAVE_GTK            "")
SET(BIAS_HAVE_GTK2           "")
SET(BIAS_HAVE_IMAGEMAGICKLIB "ON")
SET(BIAS_HAVE_IMLIB          "")
SET(BIAS_HAVE_LAPACK         "ON")
SET(BIAS_HAVE_LIBJPEG        "")
SET(BIAS_HAVE_OPENCV         "ON")
SET(BIAS_HAVE_OPENEXR        "")
SET(BIAS_HAVE_OPENGL         "ON")
SET(BIAS_HAVE_PMD            "")
SET(BIAS_HAVE_PMDUSB         "")
SET(BIAS_HAVE_PTHREADS       "ON")
SET(BIAS_HAVE_STEREOI        "")
SET(BIAS_HAVE_TIFF           "")
SET(BIAS_HAVE_UEYE           "")
SET(BIAS_HAVE_V4L            "")
SET(BIAS_HAVE_VLD            "")
SET(BIAS_HAVE_WXWIDGETS      "ON")
SET(BIAS_HAVE_XML2           "ON")
SET(BIAS_HAVE_PanTilt        "FALSE")

# optional libs compiled?
SET(BIAS_HAVE_GUI            "TRUE")
SET(BIAS_HAVE_DCAM           "")
SET(BIAS_HAVE_MATCHER2D      "1")
SET(BIAS_HAVE_NETWORKCOMM    "")
SET(BIAS_HAVE_SWISSRANGER    "")

# Image instantiations compiled?
SET(BUILD_IMAGE_UCHAR  "ON") # mandatory
SET(BUILD_IMAGE_FLOAT  "ON") # mandatory
SET(BUILD_IMAGE_INT    "ON")   # optional...
SET(BUILD_IMAGE_CHAR   "ON")
SET(BUILD_IMAGE_SHORT  "OFF")
SET(BUILD_IMAGE_USHORT "OFF")
SET(BUILD_IMAGE_UINT   "OFF")
SET(BUILD_IMAGE_DOUBLE "OFF")



# use this macro to  include BIAS VC projects
# within your own VC projects IDE solution.
# Useful (only) if you deleop with the
# Microsoft Visual Studioe (e.g. 7.1) IDE
#
# Status: works with MVSC .net 2003 7.1 and MSVS 8.0 (only).
# author Jan Woetzel 2005
#
# usage in your own CMakeLists.txt file:
#
# FIND_PACKAGE(BIAS)
# IMPORT_BIAS_PROJECTS()
#
MACRO(IMPORT_BIAS_PROJECTS)
  IF    (CMAKE_BUILD_TOOL MATCHES "(msdev|devenv)")
    ## Base libs:
    INCLUDE_EXTERNAL_MSPROJECT("BIASCommon.vcproj"          "/usr/local/Base/Common/BIASCommon.vcproj")
    INCLUDE_EXTERNAL_MSPROJECT("BIASDebug.vcproj"           "/usr/local/Base/Debug/BIASDebug.vcproj")
    INCLUDE_EXTERNAL_MSPROJECT("BIASGeometryBase.vcproj"    "/usr/local/Base/Geometry/BIASGeometryBase.vcproj")
    INCLUDE_EXTERNAL_MSPROJECT("BIASImageBase.vcproj"       "/usr/local/Base/Image/BIASImageBase.vcproj")
    INCLUDE_EXTERNAL_MSPROJECT("BIASMathBase.vcproj"        "/usr/local/Base/Math/BIASMathBase.vcproj")
    ## other libs:
    INCLUDE_EXTERNAL_MSPROJECT("BIASFeatureDetector.vcproj" "/usr/local/FeatureDetector/BIASFeatureDetector.vcproj")
    # INCLUDE_EXTERNAL_MSPROJECT("BIASFilter.vcproj"          "/usr/local/Filter/BIASFilter.vcproj")
    INCLUDE_EXTERNAL_MSPROJECT("BIASGeometry.vcproj"        "/usr/local/Geometry/BIASGeometry.vcproj")
    IF   (EXISTS "/usr/local/Gui/BIASGui.vcproj")
      INCLUDE_EXTERNAL_MSPROJECT("BIASGui.vcproj"         "/usr/local/Gui/BIASGui.vcproj")
    ENDIF(EXISTS "/usr/local/Gui/BIASGui.vcproj")
    INCLUDE_EXTERNAL_MSPROJECT("BIASImage.vcproj"           "/usr/local/Image/BIASImage.vcproj")
    IF   (EXISTS "/usr/local/Matcher2D/BIASMatcher2D.vcproj")
      INCLUDE_EXTERNAL_MSPROJECT("BIASMatcher2D.vcproj"       "/usr/local/Matcher2D/BIASMatcher2D.vcproj")
    ENDIF(EXISTS "/usr/local/Matcher2D/BIASMatcher2D.vcproj")
    INCLUDE_EXTERNAL_MSPROJECT("BIASMathAlgo.vcproj"        "/usr/local/MathAlgo/BIASMathAlgo.vcproj")
    # INCLUDE_EXTERNAL_MSPROJECT("BIASStateEstimator.vcproj"  "/usr/local/StateEstimator/BIASStateEstimator.vcproj")
    INCLUDE_EXTERNAL_MSPROJECT("BIASUtils.vcproj"           "/usr/local/Utils/BIASUtils.vcproj")
    #IF   (EXISTS "/usr/local/VideoSource/BIASVideoSource.vcproj")
    #  INCLUDE_EXTERNAL_MSPROJECT("BIASVideoSource.vcproj"     "/usr/local/VideoSource/BIASVideoSource.vcproj")
    #ENDIF(EXISTS "/usr/local/VideoSource/BIASVideoSource.vcproj")
  ENDIF (CMAKE_BUILD_TOOL MATCHES "(msdev|devenv)")
ENDMACRO(IMPORT_BIAS_PROJECTS)
