#include <Base/Common/SharedPtr.hh>

#include <iostream>

using namespace BIAS;
using namespace std;


class parent 
{
public: 
  parent() : data(0.0) {}
  parent(const double d) : data(d) {}
  virtual ~parent() {}
  double data;
  virtual void printData()
  { cout << "parent::printData: "<<data<<endl; }
};

class child : public parent
{
public:
  child() : parent() {}
  child(const double d) : parent(d) {}
  virtual ~child() {}
  virtual void printData()
  { cout << "child::printData: "<<data<<endl; }
};

ostream& operator<<(ostream& os, const parent & f)
{ os << "parent = "<<f.data; return os; }

ostream& operator<<(ostream& os, const child & f)
{ os << "child = "<<f.data; return os; }



// compile tests
void TestConstructors()
{
  SharedPtr<parent> empty;
  SharedPtr<parent> empty2(empty);
  SharedPtr<parent> filled(new parent);
  SharedPtr<parent> filled2(filled);
  SharedPtr<parent> equal = filled;
}

// compile tests
void TestAccess()
{
  const double value(2.0);
  SharedPtr<parent> sp(new parent(value));
  if ((*sp).data != value) { BIASABORT; }
  if (sp->data != value) { BIASABORT; }
  cout << "operator&: "<<&sp<<endl;
  cout << "Get: "<<Get(sp)<<endl;
  cout << "RefCount: "<<RefCount(sp)<<endl;
}

// compile tests
void TestConversionToBool()
{
  // create two SharedPtr , one filled and the other empty
  SharedPtr<parent> parent1, parent2(new parent);
  
  // test if(ptr)
   if (parent1){ BIASABORT; }
   if (parent2){} else { BIASABORT; }

  // test if (!ptr)
  if (!parent1){} else { BIASABORT; }
  if (!parent2){ BIASABORT; }
}

void TestComparison()
{
  parent *raw_1 = new parent(1.0);
  parent *raw_2 = new parent(2.0);

  child *raw_child = new child(3.0);

  SharedPtr<parent> shared_1(raw_1), shared_2(raw_2);
  SharedPtr<parent> shared_1b(shared_1), shared_2b(shared_2);
  SharedPtr<parent> shared_parent_from_raw_child(raw_child);
  
  SharedPtr<int> shared_int(new int);

  // comparison of shared pointers
  if (shared_1 == shared_1b) {} else { BIASABORT; }
  if (shared_1 != shared_1b) { BIASABORT; } 
  if (shared_1 == shared_2) { BIASABORT; }
  if (shared_1 != shared_2) {} else { BIASABORT; } 
  
  // comparison to raw pointers is done on raw pointer basis
  if (raw_1 == Get(shared_1)) {} else { BIASABORT; } 
  if (raw_1 != Get(shared_1)) { BIASABORT; }

  
  // comparison between apples and oranges, compiler error
//   if (shared_int == shared_1) { BIASABORT; }
//   if (shared_int != shared_1) {} else { BIASABORT; }

  
  // no need to cleanup, the shared pointer take care of it 
}


void TestSorting()
{
  SharedPtr<int> ptr1(new int(1));
  SharedPtr<int> ptr2(new int(2));
  
  // comparing the pointers
  if (Get(ptr1)==Get(ptr2)){ BIASABORT; }
  if (Get(ptr1)<Get(ptr2)){
    if (!(ptr1<ptr2)) { BIASABORT; }
    if (ptr1>=ptr2) { BIASABORT; }
  } else {
    if (!(ptr1>ptr2)) { BIASABORT; }
    if (ptr1<=ptr2) { BIASABORT; }
  }

  // comparing the values
  if (*ptr1>=*ptr2) { BIASABORT; }
}


void TestReferenceCounting()
{
  SharedPtr<int> ptr1;
  // new scope to test reference counting
  {
    SharedPtr<int> ptr2(new int);
    cout << "reference count is "<<RefCount(ptr2)<<" expected 1\n";
    if (RefCount(ptr2)!=1u) { BIASABORT; }
    // assign reference ptr2 to ptr1 should increas ereference counter 
    ptr1 = ptr2;
    if (RefCount(ptr2)!=2u) { BIASABORT; }
    if (RefCount(ptr1)!=2u) { BIASABORT; }
    // copy constructor should increase reference counter
    SharedPtr<int> ptr3(ptr1);
    if (RefCount(ptr2)!=3u) { BIASABORT; }
    if (RefCount(ptr1)!=3u) { BIASABORT; }
  }
  // ptr2 and ptr 3 have left their scope and desctructors should have been 
  // executed, resulting in a reference count of 1
  if (RefCount(ptr1)!=1u) { BIASABORT; }
}

void TestStream()
{
  SharedPtr<int> ptr1(new int(2));
  cout << "ptr1: "<<ptr1<<" = "<<*ptr1<<endl;
  SharedPtr<int> ptr2;
  cout << "ptr2: "<<ptr2<<endl;
}

int main()
{
  TestConstructors();
  TestAccess();
  TestConversionToBool();
  TestComparison();
  TestSorting();
  TestReferenceCounting();
  TestStream();

  return 0;
}
