#include "ThreadNaming.hh"

#include <Base/Debug/Error.hh>
#include <iostream>
#include <windows.h>

/// author Jan Woetzel, 2005


#define MS_VC_EXCEPTION 0x406d1388

// helper struct 
typedef struct tagTHREADNAME_INFO
{
  DWORD dwType;        // must be 0x1000
  LPCSTR szName;       // pointer to name (in same addr space)
  DWORD dwThreadID;    // thread ID (-1 caller thread)
  DWORD dwFlags;       // reserved for future use, most be zero
} THREADNAME_INFO;


// JW public interface 
void BIAS::SetThreadName(const unsigned long dwThreadID, const char* szThreadName )
{
  if (szThreadName==NULL) 
    return; // invalid name

  // TODO: raise exception only when running in debugger (at runtime!)
  // and additionally raise only in debug mode
  // HACK
  //BIASWARN("Thread naming temporarily disabled to avoid Exception thread on running without debug heap. dwThreadID="<<dwThreadID);
  //return;

#if !defined(_DEBUG) || !defined(BIAS_DEBUG)
  BIASWARN("Thread naming (temporarily) disabled to avoid throwing an exception when running without debug heap. dwThreadID="<<dwThreadID);
  return; // do not raise exception if running without debug heap.
#else 

  THREADNAME_INFO info;
  info.dwType = 0x1000;
  info.szName = szThreadName;
  info.dwThreadID = dwThreadID;
  info.dwFlags = 0;

  try {
    // post an exception to Visual studio to change the thread name. (JW) 
    RaiseException(
      MS_VC_EXCEPTION, 
      0, 
      sizeof(info) / sizeof(DWORD), 
      (DWORD *)&info
      );
  } 
  catch (...) 
  {
    std::cerr<<__FUNCTION__<<"not working."<<std::endl;
#  if defined(BIAS_DEBUG) && defined(_DEBUG)
    DebugBreak();
#  endif
  }
#endif
}

