/// Jan Woetzel
#include "Timing.hh"

#ifdef WIN32
#  include <stdio.h>
#  include <Windows.h>
#else
#  include <sys/time.h>
#  include <time.h>
#endif


namespace BIAS {


#ifdef WIN32

#define Li2Double(x) ((double)((x).HighPart) * 4.294967296E9 + (double)((x).LowPart))

  double Realtime (void)
  {
    LARGE_INTEGER time, freq;
    double dtime, dfreq, res;
    if (QueryPerformanceCounter(&time) == 0)
    {
      DWORD err = GetLastError();
      LPVOID buf;
      FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
        NULL,
        err,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPTSTR) &buf,
        0, NULL
        );
      printf("QueryPerformanceCounter() failed with error %d: %s\n", err, buf);
      //BIASEXIT(1);
      exit(1);
    }
    if (QueryPerformanceFrequency(&freq) == 0)
    {
      DWORD err = GetLastError();
      LPVOID buf;
      FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
        NULL,
        err,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPTSTR) &buf,
        0, NULL
        );
      printf("QueryPerformanceFrequency() failed with error %d: %s\n", err, buf);
      //BIASEXIT(1);
      exit(1);
    }
    dtime = Li2Double(time);
    dfreq = Li2Double(freq);
    res = dtime / dfreq;
    return res;
  }


#else // WIN32

  double Realtime(void)
  {
    struct timeval tv;
    gettimeofday(&tv, (struct timezone*)0);
    return ((double)tv.tv_sec + (double)tv.tv_usec / 1000000.0 );
  }

  double realtime_ (void){ 
    return realtime(); 
  }

#endif // WIN32

} // namespace

