/** @example ExampleIplWrapOpenCV.cpp
    howto use the wrapper to use a BIAS imaeg with OpenCV image algorithms.
    @author Jan Woetzel 2005 
*/
//#include <Base/Common/LeakChecking.h>

#include <iostream>
#include <string> 

// BIAS
#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>
//#include <Base/Image/ImageConvert.hh>
//#include <Base/Image/ImageDraw.hh>
#include <Base/Image/WrapBias2Ipl.hh>


#include <bias_config.h>
#ifndef BIAS_HAVE_OPENCV
#  error You need to enable OPENCV to compile this file. Please reconfigure BIAS with USE_OPENCV
#endif
#include "cv.h"
//#include "highgui.h"

using namespace std;
using namespace BIAS;



int main( int argc, char** argv ){
    bool darttest=false;
    if (argc>=2) if (strcmp(argv[1],"-darttest")==0) {
        darttest=true;
        std::cout<<"started dart test: "<<argv[0]<<std::endl; 
    }

    // load a BIAS image to work with: 
    string filename( BIAS_TESTS_DATA "r4.jpg" );
    if (!darttest && (argc>1)) filename=argv[1];
    BIAS::Image<unsigned char> img;
    if (BIAS::ImageIO::Load(filename, img) !=0){
        cout<<"error loading image "<<filename<<endl;
        return -1;
    };
    cout<<"loaded image with dim: "<<img.GetWidth()<<"x"<<img.GetHeight()<<endl;

    // wrapper to use BIAS image in OpenCV:
    WrapBias2Ipl wrap(&img);

    // work:  draw something: 
    {
        cvLine( wrap.p_imgIpl, 
            cvPoint(-20,0),
            cvPoint(200,200), 
            cvScalar(255, 0, 0),
            1, 8, 0 );
        cvEllipse( wrap.p_imgIpl, 
            cvPoint(50,100),
            cvSize(20,40),
            20,
            0, 270, 
            cvScalar(0,255,0) );

        string msg("Hello BIAS world! 1234567890");
        int linetype=CV_AA; // antialiased
        CvFont font1, font2, font3;
        cvInitFont( &font1, CV_FONT_HERSHEY_SIMPLEX, 
            1.0, 1.0,  
            0, 1, linetype);
        cvInitFont( &font2, CV_FONT_HERSHEY_PLAIN, 
            1.0, 1.0,  
            0, 1, 0);
        cvInitFont( &font3, 
            CV_FONT_HERSHEY_COMPLEX, // font name
            1.0, // hscale
            1.0, // vscale
            0,   // shear
            2,   // thickness
            linetype
            );		
        cvPutText( wrap.p_imgIpl, msg.c_str(), 
            cvPoint(100,50), &font1, cvScalar(255,0,0) );
        cvPutText( wrap.p_imgIpl, msg.c_str(), 
            cvPoint(100,100), &font2, cvScalar(0,255,0) );
        cvPutText( wrap.p_imgIpl, msg.c_str(), 
            cvPoint(100,150), &font3, cvScalar(0,0,255) );
    }

    // display using CV routines
    // color channel order may be wrong! (use fixed highgui) 
    // avoid gui display on darttest because it doesn't work on Linux without login.
    if (!darttest) wrap.Display(true, 2000);

    // output to disk: 
    //cvSaveImage("out_wrapIPL.jpg", wrap.p_imgIpl);
    BIAS::ImageIO::ExportImage("out_ExampleIplWrapOpenCV.jpg", *wrap.p_imgBias, BIAS::ImageIO::FF_jpg);

    return 0;
}
