/** @example TestMatrixIO.cpp
Test Matrix I/O precision issues.
Deprecated, these tests are now included in more general and more strict TestMatrixVectorIO.cpp
However, it doesn't harm to check twice. (JW)
@author evers */

/* 
Example Run output: 
--------------------------------------------
instance1 is:
2 3
1.251258888515884900e-003 5.635853144932401200e-001 1.933042390209662200e-001
8.087405011139255900e-001 5.850093081453902100e-001 4.798730430005798700e-001
--------------------------------------------
instance2 is:
2 3
1.251260000000000100e-003 5.635850000000000000e-001 1.933040000000000000e-001
8.087410000000000400e-001 5.850090000000000000e-001 4.798729999999999900e-001
--------------------------------------------
equality test result is: 0
*/

// file used for I/O
#ifdef WIN32
#  define TMP "tmp.txt"
#  define TMP2 "tmp.m"
#else //WIN32
#  define TMP "/tmp/tmp.txt"
#  define TMP2 "/tmp/tmp.m"
#endif //WIN32


#include <Base/Math/Matrix.hh>
#include <cmath>
#include <iomanip>
#include <iostream>
#include <string>
#include <fstream>
#include <cassert>


using namespace std; 

double randomDouble_local(){
  return ((double) rand() / (double) RAND_MAX);
}


int main(int argc, int argv[]){

  //====== INITIAL INSTANCE ======================
  //create an instance of the matrix class, and fill it with 
  // random data 
  BIAS::Matrix<double> instance1(2, 3); 

  for( int r = 0; r < 2; r++)
    for( int c = 0; c < 3; c++) 
      instance1[r][c] =  randomDouble_local(); 

  //create output file, and write to it.
  ofstream fOut(TMP, ios::out);

  fOut << instance1; 
  fOut.close();

  ofstream fmOut(TMP2, ios::out);

  instance1.WriteMatlab(fmOut, "randommat"); 
  fOut.close();

  //====== INSTANCE POPULATED FROM FILE =========  
  //open the file created
  ifstream fIn(TMP, ios::in ); 

  // create new instace of the object
  BIAS::Matrix<double> instance2(2, 3); 

  // read in from the file
  fIn >> instance2; 
  fIn.close();


  //======= TEST EQUALITY OF INSTANCES ==========   

  //test that the outputed and inputed objects pass an equality test
  bool equalityTestResult = (instance1 == instance2); 

  //show a command line user the result(s)...
  cout << setprecision(25); 
  cout << scientific; 

  cout << "--------------------------------------------\n"; 
  cout << "instance1 is: \n" << instance1 << "\n\n"; 

  cout << "--------------------------------------------\n"; 
  cout << "instance2 is: \n" << instance2 << "\n\n"; 

  cout << "--------------------------------------------\n"; 
  cout << "equality test result is: " <<boolalpha<< equalityTestResult << "\n"; 

  if (equalityTestResult == false) 
    cout << "Residuum: "<<instance2-instance1<<endl;

  //insure passes test
  BIASASSERT( equalityTestResult == true ) ;

}
