# Try to find Pont Grey Triclops 1394 firewire camera driver library. 
# For trofical stereo system and the Stereo SDK.
# available from: www.ptgrey.com
#
# Once run this will define: 
#
# 1394Triclops_FOUND
# 1394Triclops_INCLUDE_DIR       = all include directories
# 1394Triclops_LIBRARIES         = all reqwuire dlibs
# 1394Triclops_triclops_LIBRARY  =
#
# (c) Jan Woetzel 11/2007
# --------------------------------

IF (NOT WIN32)
  MESSAGE("Find1394Triclops.cmake: This library supports only WIN32. skipping.")
  SET(1394Triclops_FOUND FALSE)
ENDIF (NOT WIN32)

IF (WIN32)

  # --- find --------------------------
  
  FIND_PATH(1394Triclops_INCLUDE_DIR triclops.h
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Point Grey Research, Inc.\\Triclops;InstallDir]/include"
    "$ENV{ProgramFiles}/Point Grey Research/Triclops Stereo Vision SDK/include"    
    )

  FIND_LIBRARY(1394Triclops_triclops_LIBRARY
    NAMES triclops
    PATHS 
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Point Grey Research, Inc.\\Triclops;InstallDir]\\lib"
    "$ENV{ProgramFiles}/Point Grey Research/Triclops Stereo Vision SDK/lib"
    )


  # --- check --------------------------

  IF(1394Triclops_triclops_LIBRARY)
    SET(1394Triclops_LIBRARIES ${1394Triclops_triclops_LIBRARY})    
  ENDIF(1394Triclops_triclops_LIBRARY)

  IF(NOT 1394Triclops_INCLUDE_DIR)
    MESSAGE(SEND_ERROR "1394Triclops include dir not found.")
  ENDIF(NOT 1394Triclops_INCLUDE_DIR)

  IF(1394Triclops_LIBRARIES AND 1394Triclops_INCLUDE_DIR)
    SET(1394Triclops_FOUND TRUE)
    
    FILE(GLOB 1394Triclops_HEADER "${1394Triclops_INCLUDE_DIR}/*.h*" )
    SOURCE_GROUP(Triclops1394 FILES ${1394Triclops_HEADER})
    
    # extract link directory    
    IF    (1394Triclops_triclops_LIBRARY)
      GET_FILENAME_COMPONENT(1394Triclops_LINK_DIRECTORIES ${1394Triclops_triclops_LIBRARY} PATH)
    ENDIF (1394Triclops_triclops_LIBRARY)  
    
  ELSE(1394Triclops_LIBRARIES AND 1394Triclops_INCLUDE_DIR)
    SET(1394Triclops_FOUND FALSE)
  ENDIF(1394Triclops_LIBRARIES AND 1394Triclops_INCLUDE_DIR)

  MARK_AS_ADVANCED(
    1394Triclops_INCLUDE_DIR
    1394Triclops_triclops_LIBRARY
    1394Triclops_LIBRARIES
    1394Triclops_HEADER
    )

ENDIF (WIN32)
