/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2004 (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel

BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Filter/FilterNTo2NFactory.hh>
#include <Filter/GradientGauss.hh>
#include <Filter/GradientSimple.hh>
#include <Filter/GradientSobel3x3.hh>
#include <Filter/GradientGaussAsymmetric.hh>

using namespace BIAS;
using namespace std;

template <class InputStorageType, class OutputStorageType>
FilterNTo2N<InputStorageType, OutputStorageType>*
FilterNTo2NFactory<InputStorageType, OutputStorageType>::
Generate(const std::string& className)
{
  if (className == "GradientGauss")
    return new GradientGauss<InputStorageType,OutputStorageType>;
  if (className == "GradientSimple")
    return new GradientSimple<InputStorageType,OutputStorageType>;
  if (className == "GradientSobel3x3")
    return new GradientSobel3x3<InputStorageType,OutputStorageType>;
  if (className == "GradientGaussAsymmetric")
    return new GradientGaussAsymmetric<InputStorageType,OutputStorageType>;

  BIASERR("Invalid FilterNTo2N class: " << className);
  return NULL;
}

#define FILTER_INSTANTIATION_CLASS FilterNTo2NFactory
#define FILTER_INSTANTIATION_NO_UNSIGNED_OUTPUT
#include "Filterinst.hh"
