/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __HessianGauss_hh__
#define __HessianGauss_hh__

#include "FilterNTo3N.hh"
#include "Convolution.hh"

#include <Base/Math/Vector3.hh>

                         
namespace BIAS {

  /** @class HessianGauss
      @ingroup g_filter
      @brief gradient calculation with separated gauss masks
      @author ischiller 12/2007 */
  template <class InputStorageType, class OutputStorageType> 
  class BIASFilter_EXPORT HessianGauss : public FilterNTo3N<InputStorageType, OutputStorageType>
  {
  public:
    HessianGauss();
    HessianGauss(const HessianGauss<InputStorageType, OutputStorageType>& other);
    virtual ~HessianGauss();

    /** returns images containig hxx hyy and hxy 
        @author ischiller
        @date 12/2007*/
    virtual int Filter(const Image<InputStorageType>& src, 
                       Image<OutputStorageType>& hx,
                       Image<OutputStorageType>& hy, 
                       Image<OutputStorageType>& hxy);

    virtual FilterNTo3N<InputStorageType, OutputStorageType>* Clone() const
    { return new HessianGauss<InputStorageType, OutputStorageType>(*this); }
    
    ////////////////////////////////////////////////////////////////////////

    inline void SetHessGaussSigma(const double sigma)
    { _HessGaussSigma=sigma; }

    inline double GetHessGaussSigma() const { return _HessGaussSigma; }

    inline void SetHessGaussRatio(const double ratio)
    { _HessGaussRatio=ratio; }

    inline double GetHessGaussRatio() const { return _HessGaussRatio; }

    inline void PrintKernel() const
    { _ConvXX.PrintKernel(); }

    inline void PrintKernelXY() const
    { _ConvXY.PrintKernel(); }
  protected:
    /// the parameters
    double _HessGaussSigma, _HessGaussRatio;
    /// the parameters at the time of the last call
    double _LastSigma, _LastRatio;
    /// computation object to execute the convolution
    Convolution<InputStorageType, OutputStorageType> _ConvXX;   
    Convolution<InputStorageType, OutputStorageType> _ConvYY;   
    Convolution<InputStorageType, OutputStorageType> _ConvXY;

    /** Fills _HVec with gaussian function and _VVec with derivative of 
        gaussian function with standard deviation Sigma. The size of _HVec 
        and _VVec is calculated by using Ration. The Gaussian function is 
        truncated, when the ration between biggest and smalles entry falls
        below Ratio.
        @author ischiller 12/2007 */
    void _CalculateKernels(double Sigma, double Ratio);

    virtual void GetBordersValid_(int& border_x, int &border_y) const;

  }; // class


} // namsepace

#endif // __HessianGauss_hh__
