#ifndef __CameraViewFrustum_h__
#define __CameraViewFrustum_h__

#include <bias_config.h>
#include <iostream>

#define FRUSTUM_IS_FIXED_DEFAULT false

namespace BIAS {

  /** \brief describes internal parameter Frustum of an (OpenGL) camera view

  \author Jan Woetzel 09/2003 */
  class BIASGui_EXPORT CameraViewFrustum {
  public:
    virtual ~CameraViewFrustum(){}; ///< destructor

    /// ctors
    CameraViewFrustum();

    CameraViewFrustum(const double czNear,
      const double czFar,
      const double cleft,
      const double cright,
      const double cbottom,
      const double ctop,
      const bool   cisFixed=FRUSTUM_IS_FIXED_DEFAULT )
      :
    zNear(czNear),zFar(czFar),
      left(cleft), right(cright),
      bottom(cbottom), top(ctop),
      isFixed(cisFixed)
    {}


    /// sets a symmetric, centered frustum like gluPerspective does
    /// fovYdeg is the y field of view in degree.
    void SetPerspective(const double & fovYdeg, 
      const double & fovAspect,
      const double & zNear=1., 
      const double & zFar=1000.);

    /// @return the approximate field of view in degree
    double GetFovY() const;
    double GetFovX() const;

    /// @return aspect ratio x/y of field of image size (NOT field of view)
    double GetAspect() const;

    /// adapt left and right to match an aspect ratio
    /// useful to adapth frustum to viewport bound to windows size reshape 
    void AdaptWidth(const double & vpAspectRatioDesired );

    std::ostream & Print(std::ostream & os=std::cout ) const;

#ifdef BIAS_HAVE_OPENGL
    void DisplayGL() const;
#endif // BIAS_HAVE_OPENGL

    inline void InitMembers()
    {
      this->zNear  = 0.1;
      this->zFar   = 1000.;

      this->left   = -1. *zNear;
      this->right  = -left;
      this->bottom = -1. *zNear;
      this->top    = -bottom;

      this->isFixed=FRUSTUM_IS_FIXED_DEFAULT;
    }

  public:
    double zNear;
    double zFar;

    double left;
    double right;
    double bottom;
    double top;

    /// Should reshape adapt aspect ?
    /// or keep aspect, thus window aspect and frustum aspect may be different
    bool isFixed;

  }; // CameraViewFrustum
} // namespace BIAS
#endif
