#include <wx/wx.h>

#include <Gui/ImageCanvasEnhanced.hh>
#include <Base/Image/ImageIO.hh>
#include <Gui/StringConv.hh>
using namespace BIAS;
using namespace std;

enum { ID_Quit = 1, ID_About, ID_Load, ID_Rect};

class MyApp: public wxApp
{
  virtual bool OnInit();
};

IMPLEMENT_APP(MyApp)

class MyFrame: public wxFrame, public ICECallbackInterface
{
public:

  MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size);

  void OnQuit(wxCommandEvent& event);
  void OnAbout(wxCommandEvent& event);
  void OnLoad(wxCommandEvent& event);
  void OnRect(wxCommandEvent& event);

  virtual void RectangleSelected(int tl[2], int br[2]);

  ImageCanvasEnhanced *_ic;

  DECLARE_EVENT_TABLE()
};

bool MyApp::OnInit()
{
  MyFrame *frame = new MyFrame( AsciiToWx("ImageCanvasEnhanced"), wxPoint(50,50), 
                                wxSize(450,340) );
  frame->Show(TRUE);
  SetTopWindow(frame);
  return TRUE;
} 

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
  EVT_MENU(ID_Quit, MyFrame::OnQuit)
  EVT_MENU(ID_About, MyFrame::OnAbout)
  EVT_MENU(ID_Load, MyFrame::OnLoad)
  EVT_MENU(ID_Rect, MyFrame::OnRect)
END_EVENT_TABLE()


MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
  : wxFrame((wxFrame *)NULL, -1, title, pos, size)
{
  wxMenu *menuFile = new wxMenu;
  
  menuFile->Append( ID_About, AsciiToWx("&About...") );
  menuFile->Append( ID_Load, AsciiToWx("&Load...") );
  menuFile->Append( ID_Rect, AsciiToWx("&Rect...") );
  menuFile->AppendSeparator();
  menuFile->Append( ID_Quit, AsciiToWx("E&xit") );

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(menuFile, AsciiToWx("&File") );
  SetMenuBar(menuBar);

  CreateStatusBar(2);
  SetStatusText(AsciiToWx("ImageName"), 0);

  wxBoxSizer *box=new wxBoxSizer(wxVERTICAL);
  _ic=new ImageCanvasEnhanced(this, GetStatusBar(), 1);
  box->Add(_ic, 1, wxALL| wxGROW |wxALIGN_CENTER, 5);
  box->SetSizeHints(this);
  SetSizer(box);

  //const string fn("ucim.pgm");
  const string fn(BIAS_TESTS_DATA "r4.jpg" );
  Image<unsigned char> im;
  int res=ImageIO::Load(fn, im);
  if (res!=0){
    cout<<"could not load "<<fn<<endl;
    wxMessageBox(_T( "could not load image. aborting. ") );
    BIASABORT;
  }
  SetStatusText(_T("image"), 0);
  _ic->Show(im, "image" );
  FitInside();
}

void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{ Close(TRUE); }

void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
  wxMessageBox(AsciiToWx("This is a wxWidgtets Hello world sample"),
             AsciiToWx("About Hello World")
	       , wxOK | wxICON_INFORMATION, this);
}

void MyFrame::OnLoad(wxCommandEvent& WXUNUSED(event))
{
  wxFileDialog fileDialog(this, AsciiToWx("Choose a file"), AsciiToWx("."), 
			  AsciiToWx("ucim.pgm"));
  if (fileDialog.ShowModal() == wxID_OK){
    string fname=WxToAscii(fileDialog.GetPath());
    Image<unsigned char> im;
    if (ImageIO::Load(fname, im)!=0){
      BIASERR("error loading "<<fname);
    } else {
      SetStatusText(fileDialog.GetFilename(), 0);
      _ic->Show(im, WxToAscii(fileDialog.GetFilename()));
    }
  } else {
    cerr << "canceled loadin image" << endl;
  }
}

void MyFrame::OnRect(wxCommandEvent& WXUNUSED(event))
{
  _ic->SelectRectangle(*this);
}
 
void MyFrame::RectangleSelected(int tl[2], int br[2])
{
  cerr << "finished select rectangle: ("<<tl[0]<<", "<<tl[1]<<")  <->  ("
       <<br[0]<<", "<<br[1]<<")\n";
}
