#ifdef WIN32
// workaround for wx 2.6.x redefiniton of _CRT_SECURE_NO_DEPRECATE 
// fixed in wx 2.7.2 
#  pragma warning(disable: 4005)
#endif

#include "VideoSource_Controller.hh"

#ifdef WIN32
#  include <Base/Common/getopt_W32.h>  
#endif

#if defined(WIN32) && defined(HAVE_TRIGGER_CLASSES)
#  include "TriggerIOControl.h"
#endif

#ifdef BIAS_HAVE_DCAM
#  include <VideoSource/VideoSource_DCAM.hh>
#endif

#include <iostream>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>
#include <Gui/StringConv.hh>

using namespace std;
using namespace BIAS;

BEGIN_EVENT_TABLE(VideoSource_Controller, wxFrame)
  EVT_BUTTON(ID_HIDE,       VideoSource_Controller::OnHide)
  EVT_COMMAND_SCROLL(ID_GAIN,      VideoSource_Controller::OnGain)
  EVT_COMMAND_SCROLL(ID_SHUTTER,   VideoSource_Controller::OnShutter)
  EVT_BUTTON(ID_BUTTON_AUTO_SHUTTER, VideoSource_Controller::OnAuto_Shutter)
  EVT_BUTTON(ID_BUTTON_WHITEBALANCE, VideoSource_Controller::OnWhiteBalance)
  EVT_BUTTON(ID_BUTTON_AUTO_GAIN, VideoSource_Controller::OnAuto_Gain)
  EVT_BUTTON(ID_BUTTON_UPDATE, VideoSource_Controller::OnUpdate)
END_EVENT_TABLE()




VideoSource_Controller::
VideoSource_Controller(BIAS::VideoSource *cam, wxFrame *parent,string title,
					   double minShutter, double maxShutter,
					   double minGain,double maxGain)
  : wxFrame(parent, -1 
        ,(AsciiToWx(title.c_str()))
        ,wxDefaultPosition,wxSize(400,500),wxSTAY_ON_TOP |wxCAPTION){
  Cam_ = cam;
  minShutter_=minShutter;
  maxShutter_=maxShutter;
  minGain_= minGain;
  maxGain_= maxGain;
  Initialize();

  SetShutterRange((unsigned)minShutter_,(unsigned)maxShutter_);
  SetGainRange((unsigned)minGain_,(unsigned)maxGain_);
}



VideoSource_Controller::
~VideoSource_Controller()
{

}

void VideoSource_Controller::
Initialize(){
  //Shut the Automode DOWN
  auto_gain = false;
  auto_shutter = false;

  sliderBox_ShGa_Visible = true;    
  vBoxVisible = true;

  wxSize SlSize(70,200);
  wxSize BtSize(55,30);
  

  wxBoxSizer *slGa = new wxBoxSizer(wxVERTICAL);
  wxStaticText *textGain = new wxStaticText(this,ID_TEXT, wxT("Gain\n[dB]"));
  SliderGain = new wxSlider(this, ID_GAIN, 1500, 0, 3000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("Gain"));

  auto_ga_Button = new wxButton(this, ID_BUTTON_AUTO_GAIN, 
                                wxT("MAN"),wxDefaultPosition,BtSize);

  SliderGain->Enable(true);
  slGa->Add(textGain,0,wxALL,2);  
  slGa->Add(SliderGain,0, wxALL,2);
  slGa->Add(auto_ga_Button,0,wxALL, 2);

  wxBoxSizer *slSh = new wxBoxSizer(wxVERTICAL);
  wxStaticText *textShutter = new wxStaticText(this,ID_TEXT, wxT("Shutter\n[ms]"));
  SliderShutter = new wxSlider(this, ID_SHUTTER, 20, 1, 80,
                               wxDefaultPosition,SlSize,
                               wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                               wxDefaultValidator, wxT("Shutter"));
  
  auto_sh_Button = new wxButton(this, ID_BUTTON_AUTO_SHUTTER, 
                                wxT("MAN"),wxDefaultPosition,BtSize);
  SliderShutter->Enable(true);
  slSh->Add(textShutter,0,wxALL,2);  
  slSh->Add(SliderShutter,0,wxALL, 2);
  slSh->Add(auto_sh_Button,0,wxALL, 2);

  wxBoxSizer *sl = new wxBoxSizer(wxHORIZONTAL);
  sl->Add(slSh,0,wxALL,2);  
  sl->Add(slGa,0,wxALL, 2);

  whiteBalanceButton = new wxButton(this, ID_BUTTON_WHITEBALANCE, 
                                wxT("WB"),wxDefaultPosition,BtSize);
  sl->Add(whiteBalanceButton,0,wxALL, 2);

  showButton = new wxButton(this, ID_HIDE, wxT("Hide"),
			    wxDefaultPosition,BtSize);
  update_Button = new wxButton(this, ID_BUTTON_UPDATE, 
			       wxT("UPDATE"),wxDefaultPosition,BtSize);

  vBox = new wxBoxSizer(wxHORIZONTAL);
  vBox->Add(showButton,0,wxALIGN_LEFT   | wxBOTTOM);
  vBox->Add(update_Button, 0, wxALIGN_RIGHT   | wxBOTTOM); 

  
  sliderBox_ShGa = new wxBoxSizer(wxVERTICAL);
  sliderBox_ShGa->Add(sl,0,wxALL, 2);
  sliderBox_ShGa->Add(vBox,0,wxALL,2);

   vBox->SetSizeHints(this);
  this->SetSizer(sliderBox_ShGa);
  SetAutoLayout(true);
  Layout();
  Fit();
  wxCommandEvent fakeEvent;
  OnUpdate(fakeEvent);
  Show(TRUE);
}

void VideoSource_Controller::
SetVideoSource(BIAS::VideoSource *cam){
  Cam_ = cam;
}


void VideoSource_Controller::
OnHide(wxCommandEvent& event){
  Show(false);
}

void VideoSource_Controller::
OnClose(wxCloseEvent& event) {
  Show(false);
}

void VideoSource_Controller::
OnGain(wxScrollEvent& event) {
  double value = SliderGain->GetValue();
  if(Cam_ != NULL){
    auto_off(1);
		Cam_->SetGain(value/100.0);
  }
}

void VideoSource_Controller::
OnShutter(wxScrollEvent& event) {
  double value = SliderShutter->GetValue();
  if(Cam_ != NULL){
	  auto_off(0);
		Cam_->SetShutter(value/1000.0); 
  }
}
void VideoSource_Controller::
OnWhiteBalance(wxCommandEvent& event) {
  if(Cam_!=NULL)
    Cam_->SoftwareWhiteBalance();
}

void VideoSource_Controller::
OnAuto_Shutter(wxCommandEvent& event) {
  if(!auto_shutter) {
    auto_on(0);
  }
  else {
    auto_off(0);
  }
}

void VideoSource_Controller::
OnAuto_Gain(wxCommandEvent& event) {
  if(!auto_gain) {
    auto_on(1);
  }
  else {
    auto_off(1);
  }
}


void VideoSource_Controller::
Update() {
  if (Cam_!= NULL) {
    float current =0.0;
    current= Cam_->GetShutter();
    SliderShutter->SetValue(int(current*1000.0f));
    current = Cam_->GetGain();

    if (current <0) 
     BIASERR("Got no gain value from camera")
	else
	  SliderGain->SetValue(int(current*100));

    bool autoon = Cam_->GetAutoShutter();
    if (autoon) auto_on(0);
    else auto_off(0);
    autoon = Cam_->GetAutoGain();
    if (autoon) auto_on(1);
    else auto_off(1);
  }
}

void VideoSource_Controller::
OnUpdate(wxCommandEvent& event) {
  Update();
}




void VideoSource_Controller::
auto_on(int param) {
  //SHUTTER
  if(param == 0){
    auto_shutter = true;
    Cam_->SetAutoShutter(true);
    auto_sh_Button->SetLabel(wxT("AUTO"));
  }
  //GAIN
  else if(param == 1){
    auto_gain = true;
    Cam_->SetAutoGain(true);
    auto_ga_Button->SetLabel(wxT("AUTO"));
  }
}

void VideoSource_Controller::
auto_off(int param) {
  //SHUTTER
  if(param == 0){
    auto_shutter = false;
    Cam_->SetAutoShutter(false);
    auto_sh_Button->SetLabel(wxT("MAN"));
  }
  //GAIN
  else if(param == 1){
    auto_gain = false;
    Cam_->SetAutoGain(false);
    auto_ga_Button->SetLabel(wxT("MAN"));  
  }
}

void  VideoSource_Controller::
SetShutterRange(unsigned int min,unsigned int max)
{
  SliderShutter->SetRange(min,max);

}

void  VideoSource_Controller::
SetGainRange(unsigned int min,unsigned int max)
{
 SliderGain->SetRange(min,max);
}
