#ifdef WIN32
// workaround for wx 2.6.x redefiniton of _CRT_SECURE_NO_DEPRECATE 
// fixed in wx 2.7.2 
#  pragma warning(disable: 4005)
#endif

#include "VideoSource_Controller_SwissRanger.hh"

#ifdef WIN32
#  include <Base/Common/getopt_W32.h>  
#endif

#include <iostream>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>
#include <Gui/StringConv.hh>

using namespace std;
using namespace BIAS;

BEGIN_EVENT_TABLE(VideoSource_Controller_SwissRanger, wxFrame)
  EVT_BUTTON(ID_HIDE,       VideoSource_Controller_SwissRanger::OnHide)
  EVT_COMMAND_SCROLL(ID_AMP,      VideoSource_Controller_SwissRanger::OnAmplitude)
  EVT_COMMAND_SCROLL(ID_SAT,      VideoSource_Controller_SwissRanger::OnSaturation)
  EVT_COMMAND_SCROLL(ID_MOD,      VideoSource_Controller_SwissRanger::OnModFreq)
  EVT_COMMAND_SCROLL(ID_DIST,     VideoSource_Controller_SwissRanger::OnDistance)
  EVT_COMMAND_SCROLL(ID_INT,      VideoSource_Controller_SwissRanger::OnIntegration)
  EVT_BUTTON(ID_BUTTON_UPDATE, VideoSource_Controller_SwissRanger::OnUpdate)
END_EVENT_TABLE()




VideoSource_Controller_SwissRanger::
VideoSource_Controller_SwissRanger(BIAS::VideoSource_SwissRanger *cam, wxFrame *parent)
  : wxFrame(parent, -1, _T("VideoSourceController MultiCam")
  ,wxDefaultPosition,wxSize(500,500),wxSTAY_ON_TOP |wxCAPTION)
{
  Cam_ = cam;
  Initialize();

  min_amplitudeThreshold_=0; max_amplitudeThreshold_=2000;
  min_saturationThreshold_=0; max_saturationThreshold_=2000;
  min_integrationTime_=0; max_integrationTime_=0xFF;
  min_distanceOffset_=0; max_distanceOffset_=0xFF;
  min_modulationFrequency_=0; max_modulationFrequency_=3;


  SetVideoSource(cam);
  SetAmplitudeThresholdRange(min_amplitudeThreshold_, max_amplitudeThreshold_);
  SetSaturationThresholdRange(min_saturationThreshold_, max_saturationThreshold_);
  SetModulationRange(min_modulationFrequency_, max_modulationFrequency_);
  SetDistanceOffsetRange(min_distanceOffset_, max_distanceOffset_);
  SetIntegrationTimeRange(min_integrationTime_, max_integrationTime_);
}



VideoSource_Controller_SwissRanger::
~VideoSource_Controller_SwissRanger()
{

}

void VideoSource_Controller_SwissRanger::
Initialize(){
  
  sliderBox_ShGa_Visible = true;    
  vBoxVisible = true;

  wxSize SlSize(100,200);
  wxSize BtSize(55,30);
  
  wxBoxSizer *slAmp = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tAmp = new wxStaticText(this,ID_TEXT, wxT("Amplitude Threshold"));

  SliderAmplitude = new wxSlider(this, ID_AMP, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("Amp"));

  SliderAmplitude->Enable(true);
  slAmp->Add(tAmp,0,wxALL,2);  
  slAmp->Add(SliderAmplitude,0, wxALL,2);
  
  wxBoxSizer *slSat = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tSat = new wxStaticText(this,ID_TEXT, wxT("Saturation Threshold"));
  SliderSaturation = new wxSlider(this, ID_SAT, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("Sat"));

  SliderSaturation->Enable(true);
  slSat->Add(tSat,0,wxALL,2);  
  slSat->Add(SliderSaturation,0, wxALL,2);
  
  wxBoxSizer *slInt = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tInt = new wxStaticText(this,ID_TEXT, wxT("Integration Time"));
  SliderIntegration = new wxSlider(this, ID_INT, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("Int"));

  SliderIntegration->Enable(true);
  slInt->Add(tInt,0,wxALL,2);  
  slInt->Add(SliderIntegration,0, wxALL,2);
  
  wxBoxSizer *slMod = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tMod = new wxStaticText(this,ID_TEXT, wxT("Modulation Freq"));
  SliderModCoeff = new wxSlider(this, ID_MOD, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("ModFreq"));

  SliderModCoeff->Enable(true);
  slMod->Add(tMod,0,wxALL,2);  
  slMod->Add(SliderModCoeff,0, wxALL,2);
  
  wxBoxSizer *slDist = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tDist = new wxStaticText(this,ID_TEXT, wxT("Distance Offset"));
  SliderDistanceOffset = new wxSlider(this, ID_DIST, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("Dist"));

  SliderDistanceOffset->Enable(true);
  slDist->Add(tDist,0,wxALL,2);  
  slDist->Add(SliderDistanceOffset,0, wxALL,2);
  

  showButton = new wxButton(this, ID_HIDE, wxT("Hide"),
			    wxDefaultPosition,BtSize);
  update_Button = new wxButton(this, ID_BUTTON_UPDATE, 
			       wxT("UPDATE"),wxDefaultPosition,BtSize);

  vBox = new wxBoxSizer(wxHORIZONTAL);
  vBox->Add(showButton,0,wxALIGN_LEFT   | wxBOTTOM);
  vBox->Add(update_Button, 0, wxALIGN_RIGHT   | wxBOTTOM); 

  wxBoxSizer *sl = new wxBoxSizer(wxHORIZONTAL);
  sl->Add(slAmp,0,wxALL, 2);
  sl->Add(slSat,0,wxALL,2); 
  sl->Add(slMod,0,wxALL,2); 
  sl->Add(slInt,0,wxALL,2); 
  sl->Add(slDist,0,wxALL,2); 
  
  sliderBox_ShGa = new wxBoxSizer(wxVERTICAL);
  sliderBox_ShGa->Add(sl,0,wxALL, 2);
  sliderBox_ShGa->Add(vBox,0,wxALL,2);

  vBox->SetSizeHints(this);
  this->SetSizer(sliderBox_ShGa);
  SetAutoLayout(true);
  Layout();
  Fit();
  wxCommandEvent fakeEvent;
 // OnUpdate(fakeEvent);
  Show(TRUE);
}

void VideoSource_Controller_SwissRanger::
SetVideoSource(BIAS::VideoSource_SwissRanger *cam){
  Cam_ = cam;
}


void VideoSource_Controller_SwissRanger::
OnHide(wxCommandEvent& event){
  Show(false);
}

void VideoSource_Controller_SwissRanger::
OnClose(wxCloseEvent& event) {

  Show(false);
}

void VideoSource_Controller_SwissRanger::
OnAmplitude(wxScrollEvent& event) {
  double value = SliderAmplitude->GetValue();
  if (Cam_!= NULL) {
    Cam_->SetAmplitudeThreshold(value);		
  }
}

void VideoSource_Controller_SwissRanger::
OnSaturation(wxScrollEvent& event) {
  double value = SliderSaturation->GetValue();
  if (Cam_!= NULL) {
    Cam_->SetSaturationThreshold(value);		
  }
}

void VideoSource_Controller_SwissRanger::
OnIntegration(wxScrollEvent& event) {
  double value = SliderIntegration->GetValue();
  if (Cam_!= NULL) {
    Cam_->SetIntegrationTime(value);		
  }
}

void VideoSource_Controller_SwissRanger::
OnModFreq(wxScrollEvent& event) {
  double value = SliderModCoeff->GetValue();
  if (Cam_!= NULL) {
    Cam_->SetModulationFrequency(value);		
  }
}

void VideoSource_Controller_SwissRanger::
OnDistance(wxScrollEvent& event) {
  double value = SliderDistanceOffset->GetValue();
  if (Cam_!= NULL) {
    Cam_->SetDistanceOffset(value);		
  }
}



void VideoSource_Controller_SwissRanger::
Update() {
  if (Cam_!= NULL) {
    unsigned short currentShort =0;
    unsigned char currentChar = 0;
		Cam_->GetAmplitudeThreshold(currentShort);
    SliderAmplitude->SetValue(int(currentShort));
		Cam_->GetSaturationThreshold(currentShort);
    SliderSaturation->SetValue(int(currentShort));
    Cam_->GetIntegrationTime(currentChar);
    SliderIntegration->SetValue(int(currentChar));
 		Cam_->GetDistanceOffset(currentChar);
    SliderDistanceOffset->SetValue(int(currentChar));
 		Cam_->GetModulationFrequency(currentChar);
    SliderModCoeff->SetValue(int(currentChar));
  }
   
}

void VideoSource_Controller_SwissRanger::
OnUpdate(wxCommandEvent& event) {
  Update();
}

void  VideoSource_Controller_SwissRanger::
SetAmplitudeThresholdRange(unsigned int min,unsigned int max)
{
  SliderAmplitude->SetRange(min,max);

}

void  VideoSource_Controller_SwissRanger::
SetSaturationThresholdRange(unsigned int min,unsigned int max)
{
  SliderSaturation->SetRange(min,max);

}

void  VideoSource_Controller_SwissRanger::
SetModulationRange(unsigned int min,unsigned int max)
{
 SliderModCoeff->SetRange(min,max);
}

void  VideoSource_Controller_SwissRanger::
SetDistanceOffsetRange(unsigned int min,unsigned int max)
{
 SliderDistanceOffset->SetRange(min,max);
}

void  VideoSource_Controller_SwissRanger::
SetIntegrationTimeRange(unsigned int min,unsigned int max)
{
 SliderIntegration->SetRange(min,max);
}


