#ifndef __VideoSource_Controller_SWISSRANGER_HH__
#define __VideoSource_Controller_SWISSRANGER_HH__

#include <bias_config.h>

#include <Base/Common/W32Compat.hh>
#include <VideoSource/VideoSource_Base.hh>

#ifdef BIAS_HAVE_SWISSRANGER
#include <VideoSource/VideoSource_SwissRanger.hh>
#endif

#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>

#include <Image/Camera.hh>
#include <Image/StereoRedGreen.hh>

#ifndef BIAS_HAVE_WXWIDGETS
#  error You need BIAS_HAVE_WXWIDGETS. Recompile BIAS with USE_WXWIDGETS.
#endif

#include <wx/wx.h>



namespace BIAS {

class BIASGui_EXPORT VideoSource_Controller_SwissRanger : public ::wxFrame{
public:

  VideoSource_Controller_SwissRanger(BIAS::VideoSource_SwissRanger *cam, wxFrame *parent);

  ~VideoSource_Controller_SwissRanger();

  void Initialize();

  void SetVideoSource(BIAS::VideoSource_SwissRanger *cam);
  void SetAmplitudeThresholdRange(unsigned int min,unsigned int max);
  void SetSaturationThresholdRange(unsigned int min,unsigned int max);
  void SetModulationRange(unsigned int min,unsigned int max);
  void SetDistanceOffsetRange(unsigned int min,unsigned int max);
  void SetIntegrationTimeRange(unsigned int min,unsigned int max);
  void Update(); // trigger an update, useful for auto-mode
  

  wxBoxSizer        *vBox;
    
  enum {
    ID_HIDE,
    ID_AMP,
    ID_SAT,
    ID_MOD,
    ID_DIST,
    ID_INT,
    ID_BUTTON_UPDATE,
    ID_TEXT
  };

private:

  void OnEscape(wxCommandEvent& event);
  void OnHide(wxCommandEvent& event);
  void OnClose(wxCloseEvent& event);
  void OnAmplitude(wxScrollEvent& event);
  void OnSaturation(wxScrollEvent& event);
  void OnModulation(wxScrollEvent& event);
  void OnDistance(wxScrollEvent& event) ;
  void OnIntegration(wxScrollEvent& event) ;
  void OnModFreq(wxScrollEvent& event) ;
  void OnUpdate(wxCommandEvent& event) ;

protected:
  
  BIAS::VideoSource_SwissRanger *Cam_;
  
  wxSlider          *SliderAmplitude;
  wxSlider          *SliderSaturation;
  wxSlider          *SliderIntegration;
  wxSlider          *SliderDistanceOffset;
  wxSlider          *SliderModCoeff;
  wxBoxSizer        *sliderBox_ShGa;
  bool               sliderBox_ShGa_Visible;
 
  bool               vBoxVisible;

  wxButton          *showButton;
  wxButton          *update_Button;

   unsigned short min_amplitudeThreshold_, max_amplitudeThreshold_;
   unsigned short min_saturationThreshold_, max_saturationThreshold_;
   unsigned char min_integrationTime_, max_integrationTime_;
   unsigned char min_distanceOffset_, max_distanceOffset_;
   int min_modulationFrequency_, max_modulationFrequency_;

  DECLARE_EVENT_TABLE()

private:
    };

}
#endif
