/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BIASpragmaStart.hh>

#ifndef __DisplacementMapping_hh__
#define __DisplacementMapping_hh__

#include <Image/BackwardMapping.hh>



namespace BIAS {

  /** @class DisplacementMapping
      @brief Maps image src to image sink with displacement mapl generated
      by backwardmapping method     

      TO-DO: - Displacement MAP Currently CONTAINS BIAS-PIXEL-COORDINATES
               they must be converted into TEXTURE-COORDINATES 

      @author amattal 05/2007
  */ 
  template <class InputStorageType, class OutputStorageType> 
  class BIASImage_EXPORT DisplacementMapping 
    : public  BackwardMapping<InputStorageType, OutputStorageType>  {
  public:

    virtual ~DisplacementMapping();

    DisplacementMapping();
    DisplacementMapping(Image<float>& dismap);

    /**  required because assignment is */
    DisplacementMapping(const DisplacementMapping & src) {
      (*this) = src;
    }

    /**  required because of const members */
    DisplacementMapping<InputStorageType, OutputStorageType> & 
    operator=(const DisplacementMapping & src) {
      BIASERR("dont copy me - not implemented."); 
      BIASABORT;
      return *this;
    }

    int SetDisplacementMap(Image<float>& dismap);
 
    /** @brief reimplementation for homography, takes sink and computes 
        coords in source taking the values from displacement map dismap_*/
    virtual int GetSourceCoordinates_(const HomgPoint2D& sink,
                                      HomgPoint2D& source) const;

  protected:
    //says if the dismap_ is filled
    bool          mapped_;

    //Displacement Map of the Displacement Mapper
    //          *****************
    //should be *  interleaved  *
    //          * 3-channel RGB *
    //          *    flipped    *  
    //          *  float-image  *      with values as follows:
    //          *****************
    //dismap_(x,y) = (xx,yy,zz)
    //(xx,yy) - corresponding unmapped TEXTUREcoordinates of sinkpoint (x,y) 
    //          in the source image
    // zz = -1.0 - unmapped pixel is not within borders of the image
    //    =  1.0 - unmapped pixel is within the image,
    //             unmapped coordinates of which are (xx,yy)
    Image<float>  dismap_;

    using BackwardMapping<InputStorageType, OutputStorageType>::offsetX_;
    using BackwardMapping<InputStorageType, OutputStorageType>::offsetY_;

  };


 

} // namespace 

#endif

#include <Base/Common/BIASpragmaEnd.hh>
