/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <Base/Image/ImageIO.hh>
#include "ExampleMixtureOfGaussiansFrame.hh"

using namespace BIAS;
using namespace std;

BEGIN_EVENT_TABLE(ExampleMixtureOfGaussiansFrame, wxFrame)
  EVT_MENU      (ID_Quit,               ExampleMixtureOfGaussiansFrame::OnQuit)
  EVT_MENU      (ID_Grab,               ExampleMixtureOfGaussiansFrame::OnGrab)
  EVT_MENU      (ID_GrabContinuous,     ExampleMixtureOfGaussiansFrame::OnGrabContinuous) 
  EVT_MENU      (ID_Show1,              ExampleMixtureOfGaussiansFrame::OnShowImage1) 
  EVT_MENU      (ID_Show2,              ExampleMixtureOfGaussiansFrame::OnShowImage2) 
  EVT_MENU      (ID_Show3,              ExampleMixtureOfGaussiansFrame::OnShowImage3) 
  EVT_MENU      (ID_Show4,              ExampleMixtureOfGaussiansFrame::OnShowImage4) 
  EVT_MENU      (ID_Show5,              ExampleMixtureOfGaussiansFrame::OnShowImage5) 
  EVT_MENU      (ID_Show6,              ExampleMixtureOfGaussiansFrame::OnShowImage6) 
  EVT_MENU      (ID_Show7,              ExampleMixtureOfGaussiansFrame::OnShowImage7) 
  EVT_MENU      (ID_Show8,              ExampleMixtureOfGaussiansFrame::OnShowImage8)
  EVT_MENU      (ID_Show9,              ExampleMixtureOfGaussiansFrame::OnShowImage9)   
  EVT_MENU      (ID_Controller,         ExampleMixtureOfGaussiansFrame::OnShowController)
  EVT_TIMER     (ID_Timer,              ExampleMixtureOfGaussiansFrame::OnTimer)
  
  EVT_KEY_DOWN  (ExampleMixtureOfGaussiansFrame::OnKey) 
END_EVENT_TABLE()


ExampleMixtureOfGaussiansFrame::ExampleMixtureOfGaussiansFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
  : wxFrame(NULL, -1, title, pos, size) {

  // all parameters for MOG here, change if desired
  mog_ = new MixtureOfGaussians(4, 3, 0.2f );
  mog_->InitWeights(40, 20, 2.9f );
  roi_.UL.x = 220;
  roi_.UL.y = 75;
  roi_.LR.x = 1325;
  roi_.LR.y = 1180;
  downsample_ = true;
  
  
  // create status bar and add status text
  CreateStatusBar(2);
  wxString text;
  text.Printf(wxT("size: %4dx%4d"),0, 0);
  SetStatusText(text, 0);
  
  // init image canvas
  IC_ = new ImageCanvas(this,GetStatusBar(), 0,-1,wxDefaultPosition,                          
                        wxSize(640,480));

  // add image canvas to main window
  wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
  vbox->Add(IC_,1,  wxEXPAND |wxALL |wxFIXED_MINSIZE , 5);
  vbox->SetMinSize(640,480);
  SetSizer(vbox);
  SetAutoLayout(true);
  Layout();

  // init and start timer
  Timer_.SetOwner(this, ID_Timer);  
  Timer_.Start(10,true);
  StopWatch_.Start();  
  ShowImage_ = &CamImg_;
  dCurImage_ = 0;
}

void ExampleMixtureOfGaussiansFrame::InitCamera() {
  Camera_ = new VideoSource_DCAM();
  Camera_->OpenDevice();
  Camera_->InitImage(CamImgOrg_);
  Camera_->PreGrab();
}

void ExampleMixtureOfGaussiansFrame::CreateMenu() {
  // create file menu
  wxMenu *menuFile = new wxMenu;
  menuFile->Append(ID_Quit, wxT("E&xit") );

  // create menu for showing different cameras
  wxMenu *menuShow = new wxMenu;
  menuShow->Append(ID_Show1, wxT("Original Image") );
  menuShow->Append(ID_Show2, wxT("Normalized Image") );
  menuShow->Append(ID_Show3, wxT("Weight Image") );
  menuShow->Append(ID_Show4, wxT("Difference Image") );
  menuShow->Append(ID_Show5, wxT("Visualization") );
//  menuShow->Append(ID_Show6, wxT("Camera 6") );
//  menuShow->Append(ID_Show7, wxT("Camera 7") );
//  menuShow->Append(ID_Show8, wxT("Camera 8") );
//  menuShow->Append(ID_Show9, wxT("Camera 9") );
  menuShow->AppendSeparator();
  menuShow->Append(ID_Controller, wxT("Controller") );  
  
  // add menues to menu bar
  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(menuFile, wxT("&File"));
  menuBar->Append(menuShow, wxT("&Show"));
  SetMenuBar(menuBar);

  // fit all components to size of window
  Fit();
}

void ExampleMixtureOfGaussiansFrame::OnQuit(wxCommandEvent& event) {
  Timer_.Stop();
  Close(true);
}

void ExampleMixtureOfGaussiansFrame::OnGrab(wxCommandEvent& event) {}
void ExampleMixtureOfGaussiansFrame::OnGrabContinuous(wxCommandEvent& event) {} 
void ExampleMixtureOfGaussiansFrame::OnShowImage1(wxCommandEvent& event) {
  dCurImage_ = 0;
}
void ExampleMixtureOfGaussiansFrame::OnShowImage2(wxCommandEvent& event) {
  dCurImage_ = 1;
}
void ExampleMixtureOfGaussiansFrame::OnShowImage3(wxCommandEvent& event) {
  dCurImage_ = 2;
} 
void ExampleMixtureOfGaussiansFrame::OnShowImage4(wxCommandEvent& event) {
  dCurImage_ = 3;
}
void ExampleMixtureOfGaussiansFrame::OnShowImage5(wxCommandEvent& event) {
  dCurImage_ = 4;
}
void ExampleMixtureOfGaussiansFrame::OnShowImage6(wxCommandEvent& event) {}
void ExampleMixtureOfGaussiansFrame::OnShowImage7(wxCommandEvent& event) {}
void ExampleMixtureOfGaussiansFrame::OnShowImage8(wxCommandEvent& event) {}
void ExampleMixtureOfGaussiansFrame::OnShowImage9(wxCommandEvent& event) {}     
void ExampleMixtureOfGaussiansFrame::OnShowController(wxCommandEvent& event) {} 

void ExampleMixtureOfGaussiansFrame::OnTimer(wxTimerEvent& event) {
  // update frames per second and restart timer and counters
  Timer_.Stop();
  StopWatch_.Stop();
  double fps = 1000000.0 / StopWatch_.GetRealTime();
  StopWatch_.Reset();
  StopWatch_.Start();
  FrameCounter_++;
  wxString text;
  text.Printf(wxT("%05d %2.1f fps "),FrameCounter_, fps);
  SetStatusText(text, 1);
  Camera_->GrabSingle(CamImgOrg_);
  if(CamImgOrgRGB_.IsEmpty())
    CamImgOrgRGB_.Init(CamImgOrg_.GetWidth(), CamImgOrg_.GetHeight(),CamImgOrg_.GetChannelCount ());
  ImageConvert::Convert(CamImgOrg_, CamImgOrgRGB_, ImageBase::CM_RGB);
  CamImgOrgRGB_.SetROI(roi_.UL.x,roi_.UL.y,roi_.LR.x,roi_.LR.y);
  CamImgOrgRGB_.GetCopyOfROI(CamImg_);
  CamImgOrgRGB_.UnsetROI();

  if (downsample_) {
    if (CamImgRescaled_.IsEmpty())
      CamImgRescaled_.Init(CamImg_.GetWidth()/2,CamImg_.GetHeight()/2,3);
    Rescale<unsigned char, unsigned char> pRescaleFilter_;
    pRescaleFilter_.DownsampleBy2Color (CamImg_,CamImgRescaled_);
    CamImg_ = CamImgRescaled_;
  }

  unsigned int w,h;
  // get width and height from image
  w = CamImg_.GetWidth();
  h = CamImg_.GetHeight();      


  text.Printf(wxT("size: %4dx%4d"),w, h);
  SetStatusText(text, 0);

  // set new height or width if changed
  if (w != Width_ || h != Height_)
  {
    Width_  = w;
    Height_ = h;
    IC_->SetSize(w+50,h+80);
    SetMinSize(wxSize(w+50,h+80));
    Fit();
  }
  BIAS::Image<float> tmp;  
  mog_->Apply(CamImg_);
  
  switch (dCurImage_) {
  
  case 0:
    ShowImage_ = &CamImg_;
    break;
  case 1:
    mog_->GetNormalizedImage(tmp);
    tmp.ScaleShiftBetween(0,255);
    BIAS::ImageConvert::ConvertST(tmp, CamImgNormalizedChar_, ImageBase::ST_unsignedchar);
    ShowImage_ = &CamImgNormalizedChar_;
    break;
  case 2:
    mog_->GetWeightImage(tmp);
    tmp.ScaleShiftBetween(0,255);
    BIAS::ImageConvert::ConvertST(tmp, WeightImgChar_, ImageBase::ST_unsignedchar);
    ShowImage_ = &WeightImgChar_;
    break;
  case 3:
    mog_->GetDifferenceImage(DifferenceImgChar_);
    ShowImage_ = &DifferenceImgChar_;
    break;
  case 4:
    mog_->GetDifferenceImageWithVisuals(DifferenceImgCharRGB_);
    ShowImage_ = &DifferenceImgCharRGB_;
    break;
  default:
    ShowImage_ = &CamImg_;
    break;
  }

  // show actual camera image on main frame
  IC_->Show(*ShowImage_, "Camera Image");

  Timer_.Start(10, true);  
}

void ExampleMixtureOfGaussiansFrame::OnKey(wxKeyEvent& event) {}
