/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __EXAMPLE_MIXTURE_OF_GAUSSIANS_FRAME_HH__
#define __EXAMPLE_MIXTURE_OF_GAUSSIANS_FRAME_HH__

#include <Base/Common/BIASpragmaStart.hh>
#include <bias_config.h>

#include <Gui/StringConv.hh>
#include <Gui/ImageCanvas.hh>
#include <Image/Camera.hh>
#include <Image/MixtureOfGaussians.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Debug/TimeMeasure.hh>
#include <Filter/Rescale.hh>

#include <VideoSource/VideoSource_Base.hh>
#include <Gui/VideoSource_Controller.hh>
#ifdef BIAS_HAVE_DCAM
#  include <VideoSource/VideoSource_DCAM.hh>
#endif // BIAS_HAVE_DCAM

#include <wx/wx.h>
#include <wx/toolbar.h> 

namespace BIAS {

  // event ids
  enum {
    ID_Quit = 42,
    ID_Timer,
    ID_Grab,
    ID_GrabContinuous,    
    ID_Key,
    ID_Show1, 
    ID_Show2,
    ID_Show3,
    ID_Show4,
    ID_Show5,
    ID_Show6,
    ID_Show7,
    ID_Show8,
    ID_Show9,    
    ID_Controller
  };  

  /** @brief example for usage of MixtureOfGaussians scene change detection
   *   example for usage of MixtureOfGaussians scene change detection. see constructor for 
   *   parameters used in this example. it has been tested with point grey grasshopper camera
   *   and a region of interest has been set specifically for this.
   *  @author fkellner 
   */
  class ExampleMixtureOfGaussiansFrame : public wxFrame
  {
  public:

    ExampleMixtureOfGaussiansFrame(const wxString& title, const wxPoint& pos,
                   const wxSize& size);
    virtual ~ExampleMixtureOfGaussiansFrame(){};

    /** Initialize cameras with param file. Only call after InitParams().
     */
    void InitCamera();

    /** Creates the menu of the main window.
     */
    void CreateMenu();

    /** Stores one image from all activated cameras
     */
    void GrabImage();
    
  protected:
  
    // functions for event handling
    void OnQuit(wxCommandEvent& event);
    void OnGrab(wxCommandEvent& event);
    void OnGrabContinuous(wxCommandEvent& event); 
    void OnShowImage1(wxCommandEvent& event); 
    void OnShowImage2(wxCommandEvent& event); 
    void OnShowImage3(wxCommandEvent& event); 
    void OnShowImage4(wxCommandEvent& event); 
    void OnShowImage5(wxCommandEvent& event); 
    void OnShowImage6(wxCommandEvent& event); 
    void OnShowImage7(wxCommandEvent& event); 
    void OnShowImage8(wxCommandEvent& event);
    void OnShowImage9(wxCommandEvent& event);     
    void OnShowController(wxCommandEvent& event); 
    void OnTimer(wxTimerEvent& event);  
    void OnKey(wxKeyEvent& event);

    unsigned int FrameCounter_;
    unsigned int Width_, Height_;
    unsigned int dCurImage_;

    BIAS::VideoSource_DCAM *Camera_;
    // camera images
    BIAS::Camera<unsigned char> CamImgOrg_;
    BIAS::Camera<unsigned char> CamImgOrgRGB_;
    BIAS::Camera<unsigned char> CamImgRescaled_;
    BIAS::Camera<unsigned char> CamImg_;
    BIAS::Camera<unsigned char> CamImgNormalizedChar_;
    BIAS::Camera<unsigned char> WeightImgChar_;
    BIAS::Camera<unsigned char> DifferenceImgChar_;
    BIAS::Camera<unsigned char> DifferenceImgCharRGB_;
    BIAS::Camera<float> CamImgNormalized_;

    BIAS::Camera<unsigned char> *ShowImage_;    

    BIAS::ImageCanvas* IC_;
    
    wxTimer Timer_;
    TimeMeasure StopWatch_; 
    
    BIAS::MixtureOfGaussians *mog_;
    BIAS::MOGHelper::Corners roi_;
    bool downsample_;
    
    DECLARE_EVENT_TABLE()           
  }; // end of class
    
} // end of namespace MIP

#include <Base/Common/BIASpragmaEnd.hh>

#endif // __EXAMPLE_MIXTURE_OF_GAUSSIANS_FRAME_HH__
