/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <Image/PyramidImage.hh>
  //#include <Image/NewPyramidImage2.hh>
#include <Base/Debug/Debug.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Debug/TimeMeasure.hh>

#include <sstream>
#include <iomanip>

using namespace BIAS;
using namespace std;

#define type 1
#if type == 1
#define StorageType unsigned char
#define KernelType int
#else 
#define StorageType float
#define KernelType float
#endif

int WritePyImage(const PyramidImage<StorageType>& pim,
                 const std::string& name="pyim")
{
  ostringstream os;
  vector<double> fac=pim.GetFactors();
  for (unsigned int i=0; i<pim.size(); i++){
    os.str("");
    os << name<<"-"<<setw(2)<<setfill('0')<<i<<"-"<<fac[i]<<".mip";
    if (ImageIO::Write(os.str(), *pim[i])!=0){
      BIASERR("error writing file "<<os.str());
      return -3;
    }
  } 

  Image<StorageType> im;
  pim.GetSingleImage(im);
  os.str("");
  os << name<<"-all.mip";
  ImageIO::Write(os.str(), im);

  return 0;
}



int main(int argc, char *argv[])
{
  TimeMeasure timer;
  Image<unsigned char> ucim;
  Image<StorageType> im;
  PyramidImage<StorageType> pyr, pyr2;
  unsigned int count=10;
  ostringstream os;

  int argind = 1;

  if (argc-argind<1 || argind<1){
    cerr << argv[0] << " [parameter] <im1> <im2> [ <im3> ... ] \n";
    return -2;
  } 
  
  for (int i=argind; i<argc; i++){
#if type == 1
    if (ImageIO::Load(argv[i], im)!=0){
#else
    if (ImageIO::Load(argv[i], ucim)!=0){
#endif
      BIASERR("error loading image "<<argv[i]);
      return -1;
    } else {
      cerr << "read "<<argv[i]<<endl;
    }
  
#if type != 1
    if (ImageConvert::ConvertST(ucim, im, ImageBase::ST_float)!=0){
      BIASERR("error converting image "<<argv[i]);
    }
#endif

    timer.Start();
    for (unsigned int k=0; k<count; k++){
      if (!pyr.IsEmpty()) pyr.Release();
      pyr.Init(im);
    }
    timer.Stop();
    cout << "calculation of PyramidImage size "<<pyr.size()
         <<" of an "<<im.GetWidth()<<"x"<<im.GetHeight()<<" Image took "
         <<timer.GetUserTime()/count<<" ms"<<endl;

    os.str("");
    os << "PyramidImage-"<<setw(3)<<setfill('0')<<i;
    if (WritePyImage(pyr, os.str())!=0){
      BIASERR("error writing "<<os.str());
    }

    pyr2.Init(im.GetWidth(), im.GetHeight(), im.GetChannelCount());
    
    timer.Reset();
    timer.Start();
    for (unsigned int k=0; k<count; k++){
      memcpy((void *)pyr2[0]->GetImageData(), 
             (void *)im.GetImageData(), 
             sizeof(StorageType)*im.GetPixelCount());
      if (pyr2.Downsample()!=0){
        BIASERR("error in Downsample()");
      }
    }
    timer.Stop();
    cout << "calculation of PyramidImage size "<<pyr2.size()
         <<" of an "<<im.GetWidth()<<"x"<<im.GetHeight()<<" Image took "
         <<timer.GetUserTime()/count<<" ms"<<endl;

    os.str("");
    os << "PyramidImage2-"<<setw(3)<<setfill('0')<<i;
    if (WritePyImage(pyr2, os.str())!=0){
      BIASERR("error writing "<<os.str());
    }
    
  }
 
  return 0;
}
