/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __HomographyRectification_hh__
#define __HomographyRectification_hh__

#include <Image/Camera.hh>
#include <Base/Image/Image.hh>
#include <Geometry/ProjectionParametersBase.hh>
#include <Image/RectificationBase.hh>

namespace BIAS {
  template <class InputStorageType, class OutputStorageType> 
  class BIASImage_EXPORT HomographyRectification : 
    public RectificationBase<InputStorageType, OutputStorageType> {
  public:
    /** Rectify with same internal parameters for both**/
    virtual int Rectify();
    /** Rectify so that images have minimal size **/
    virtual int RectifyMinImageSize();
    virtual int Disp2Depth(const Image<float>& DisparityMap, 
                           Image<float>& DepthMap, 
                           unsigned int border);
    virtual bool 
    IsInputCameraValid(const BIAS::Image<InputStorageType>& img,
                       const BIAS::ProjectionParametersBase* proj);

  protected:
    
    
  };
} //end of namespace


#endif

