/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "RegionMatcher.hh"

using namespace BIAS;
using namespace std;

#define INSTUC(uctype)\
template BIASMatcher2D_EXPORT void RegionMatcher::NCC<uctype>(double p1[2],\
  double p2[2], uctype **ida1, uctype **ida2, unsigned halfwinsize,\
  double& result);\
template BIASMatcher2D_EXPORT void RegionMatcher::NCC<uctype>(\
  unsigned int p1[2], unsigned int p2[2], uctype **ida1, uctype **roi1,\
  uctype **ida2, uctype **roi2, unsigned hww, unsigned hwh, double& result);\
template BIASMatcher2D_EXPORT int RegionMatcher::NCCSearchOdd<uctype>(\
  unsigned int p1[2], unsigned int p2[2], uctype **ida1, uctype **ida2,\
  unsigned int halfnccwinsize, unsigned int halfsearchwinsize,\
  unsigned int resultpoint[2], double mincorrelation, double& result);\
template BIASMatcher2D_EXPORT int RegionMatcher::NCCSearchEven<uctype>(\
  unsigned int p1[2], unsigned int p2[2],uctype **ida1, uctype **ida2,\
  unsigned int halfnccwinsize, unsigned int halfsearchwinsize,\
  unsigned int resultpoint[2], double mincorrelation, double& result);\
template BIASMatcher2D_EXPORT int RegionMatcher::LineMatcherNCC<uctype>(\
  unsigned int p1[2], unsigned int start2[2], unsigned int end2[2],\
  uctype **ida1, uctype **ida2, uctype **grad1, uctype **grad2,\
  unsigned int halfwinsize, unsigned int epsilon, double gradientscale,\
  unsigned int result[2], double& correlation);\
template BIASMatcher2D_EXPORT int RegionMatcher::LineMatcherSAD<uctype>(\
  unsigned int p1[2], unsigned int start2[2], unsigned int end2[2],\
  uctype **ida1, uctype **ida2, uctype **grad1, uctype **grad2,\
  unsigned int halfwinsize, unsigned int epsilon, double gradientscale,\
  unsigned int result[2], double& sad);\
template BIASMatcher2D_EXPORT int RegionMatcher::ParabolaNCC(\
  unsigned int p1[2], unsigned int p2[2], uctype **ida1, uctype **ida2,\
  unsigned int halfwinsize, KLT_TYPE result[2]);\
template BIASMatcher2D_EXPORT int RegionMatcher::ParabolaNCC5(\
  unsigned int p1[2], unsigned int p2[2], uctype **ida1, uctype **ida2,\
  unsigned int halfwinsize, KLT_TYPE result[2]);\
template BIASMatcher2D_EXPORT void RegionMatcher::ColorNCC<uctype>(\
  unsigned int p1[2], unsigned int p2[2], uctype **ida1, uctype **ida2,\
  unsigned int halfwinsize, double& result);\
template BIASMatcher2D_EXPORT void RegionMatcher::CNCC<uctype>(\
  unsigned int p1[2], unsigned int p2[2], uctype **ida1, uctype **ida2,\
  unsigned int halfwinsize, double& result);\

#define INST(type)\
INSTUC(type)\
template BIASMatcher2D_EXPORT void RegionMatcher::SSD<type>(unsigned int p1[2]\
  , unsigned int p2[2], type **ida1, type **ida2, unsigned int halfwinsize,\
  double& result);\
template BIASMatcher2D_EXPORT void RegionMatcher::NCC<type>(unsigned int p1[2]\
  , unsigned int p2[2], type **ida1, type **ida2, unsigned int halfwinsize,\
  double& result);\
template BIASMatcher2D_EXPORT void RegionMatcher::SAD<type>(unsigned int p1[2]\
  , unsigned int p2[2], type **ida1, type **ida2, unsigned int halfwinsize,\
  double& result);\
template BIASMatcher2D_EXPORT void RegionMatcher::SADN<type>(\
  unsigned int p1[2], unsigned int p2[2], type **ida1, type **ida2,\
  unsigned int halfwinsize, double& result);



INSTUC(unsigned char)
INST(float)

// fill in instances as required
#ifdef BUILD_IMAGE_INT
INST(int)
#endif
#ifdef BUILD_IMAGE_CHAR
INST(char)
#endif
#ifdef BUILD_IMAGE_SHORT
INST(short)
#endif
#ifdef BUILD_IMAGE_USHORT
INST(unsigned short)
#endif
#ifdef BUILD_IMAGE_UINT
INST(unsigned int)
#endif
#ifdef BUILD_IMAGE_DOUBLE
INST(double)
#endif


#include "LineMatcher.cpp"
#include "SimilarityMeasure.cpp"


RegionMatcher::RegionMatcher()
  : Debug()
{
  _klthws=_ncchws=_winsize=_nccwinsize=0; 
  _bl1_5.newsize(5, 5); 
  _bl2_5.newsize(5, 5); 
  _gx1_5.newsize(5, 5); 
  _gy1_5.newsize(5, 5); 
  _gx2_5.newsize(5, 5); 
  _gy2_5.newsize(5, 5); 
  _gsx_5.newsize(5, 5); 
  _gsy_5.newsize(5, 5);
}

RegionMatcher::~RegionMatcher()
{}



template <class StorageType>
int RegionMatcher::NCCSearchOdd(unsigned int p1[2], unsigned int p2[2], 
                                StorageType **ida1, StorageType **ida2, 
                                unsigned int halfnccwinsize,
                                unsigned int halfsearchwinsize, 
                                unsigned int resultpoint[2], 
                                double mincorrelation, double& result)
{
  int res=0; // return value
  unsigned int p3[2], p4[2];
  double tmpresult, maxresult;

  BIASDOUT(D_REGION_MATCHER_SEARCH_ODD, "\np1: "<<p1[0]<<" "<<p1[1]
           <<"\tp2: "<<p2[0]<<" "<<p2[1]<<"\tida1: "<<&ida1[0]
           <<" ida2: "<<&ida2[0]<<" halfnccwinsize: "<<halfnccwinsize);
  // initialize maxresult and point
  resultpoint[0]=p2[0];
  resultpoint[1]=p2[1];
  NCC(p1, p2, ida1, ida2, halfnccwinsize, maxresult);
  BIASDOUT(D_REGION_MATCHER_SEARCH_ODD,"("<<p2[0]<<", "<<p2[1]<<") : "
           <<maxresult);
  if (maxresult > mincorrelation){
    result = maxresult;
    BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "first correlation succeeded "
             <<result);
  } else {
    BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "first correlation "<<maxresult);
    for (register unsigned int sr=1; sr<=halfsearchwinsize; sr++){
      BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "search radius " << sr);
      p3[1]=p2[1]-sr;
      p4[1]=p2[1]+sr;
      for (p3[0]=p2[0]-sr, p4[0]=p2[0]-sr; p3[0]<=p2[0]+sr; 
           p3[0]++, p4[0]++){
        NCC(p1, p3, ida1, ida2, halfnccwinsize, tmpresult);
        BIASDOUT(D_REGION_MATCHER_SEARCH_ODD,"("<<p3[0]<<", "<<p3[1]<<") : "
                 <<tmpresult);
        if (tmpresult>maxresult){
          maxresult = tmpresult;
          resultpoint[0]=p3[0]; resultpoint[1]=p3[1];
        }
        if (maxresult>mincorrelation) break;

        NCC(p1, p4, ida1, ida2, halfnccwinsize, tmpresult);
        BIASDOUT(D_REGION_MATCHER_SEARCH_ODD,"("<<p4[0]<<", "<<p4[1]<<") : "
                 <<tmpresult);
        if (tmpresult>maxresult){
          maxresult = tmpresult;
          resultpoint[0]=p4[0]; resultpoint[1]=p4[1];
        }
        if (maxresult>mincorrelation) break;
      } 
      if (maxresult<result){
        p3[0]=p2[0]-sr;
        p4[0]=p2[0]+sr;
        for (p3[1]=p2[1]-sr-1, p4[1]=p2[1]-sr-1; p3[1]<=p2[1]+sr-1; 
             p3[1]++, p4[1]++){
          NCC(p1, p3, ida1, ida2, halfnccwinsize, tmpresult);
          BIASDOUT(D_REGION_MATCHER_SEARCH_ODD,"("<<p3[0]<<", "<<p3[1]<<") : "
                   <<tmpresult);
          if (tmpresult>maxresult){
            maxresult = tmpresult;
            resultpoint[0]=p3[0]; resultpoint[1]=p3[1];
          }
          if (maxresult>mincorrelation) break;

          NCC(p1, p4, ida1, ida2, halfnccwinsize, tmpresult);
          BIASDOUT(D_REGION_MATCHER_SEARCH_ODD,"("<<p4[0]<<", "<<p4[1]<<") : "
                   <<tmpresult);
          if (tmpresult>maxresult) {
            maxresult = tmpresult;
            resultpoint[0]=p4[0]; resultpoint[1]=p4[1];
          }
          if (maxresult>mincorrelation) break;
        } 
      }
      if (maxresult>mincorrelation){
        result = maxresult;
        BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "succeeded at searchradius "<<sr
                 <<" with "<<result<<" correlation point: "<<resultpoint);
        break;
      }
    } // for (sr=1; sr<=halfsearchwinsize; sr++)
    result=maxresult;
    if (mincorrelation < 1.0 && result < mincorrelation){
      BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "no success within "<<p2[0] <<" "
               << p2[1]
               <<" +/- "<<halfsearchwinsize<<"  max correlation "<<result
               <<" at "<<resultpoint<< " still lower than "<<mincorrelation);
      res=-3;
    } else if (mincorrelation >= 1.0){
      BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "searched within "<<p2[0]<<" "
               <<p2[1]
               <<" +/- "<<halfsearchwinsize<<" found max correlation "<<result
               <<" at "<<resultpoint[0]<<", "<<resultpoint[1]);
    }
  } // if (maxresult > result)

  return res;
}


template <class StorageType>
int RegionMatcher::NCCSearchEven(unsigned int p1[2], unsigned int p2[2], 
                                StorageType **ida1, StorageType **ida2, 
                                unsigned int halfnccwinsize,
                                unsigned int halfsearchwinsize, 
                                unsigned int resultpoint[2], 
                                double mincorrelation, double& result)
{
  int res=0; // return value
  unsigned int p3[2], p4[2];
  double tmpresult, maxresult=-1.0;

  /*
  // initialize maxresult and point
  resultpoint[0]=p2[0];
  resultpoint[1]=p2[1];
  NCC(p1, p2, ida1, ida2, halfnccwinsize, maxresult);
  BIASDOUT(D_REGION_MATCHER_SEARCH_EVEN,"("<<p2[0]<<", "<<p2[1]<<") : "
  <<maxresult);
  if (maxresult > mincorrelation){
  result = maxresult;
  BIASDOUT(D_REGION_MATCHER_SEARCHNCC,"first correlation succeeded "
  <<result);
  } else {
  BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "first correlation = "<<maxresult
  <<"  at "<<p2[0]<<" "<<p2[1]);
  */
  BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "\np1: "<<p1[0]<<" "<<p1[1]
           <<"\tp2: "<<p2[0]<<" "<<p2[1]<<"\tida1: "<<&ida1[0]
           <<" ida2: "<<&ida2[0]<<" halfnccwinsize: "<<halfnccwinsize);
  for (register unsigned int sr=1; sr<=halfsearchwinsize; sr++){
    BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "search radius " << sr);
    // top and bottom side of quadrate
    p3[1]=p2[1]-sr+1;
    p4[1]=p2[1]+sr;
    for (p3[0]=p2[0]-sr+1, p4[0]=p2[0]-sr+1; p3[0]<=p2[0]+sr; 
         p3[0]++, p4[0]++){
      NCC(p1, p3, ida1, ida2, halfnccwinsize, tmpresult);
      BIASDOUT(D_REGION_MATCHER_SEARCH_EVEN,"("<<p3[0]<<", "<<p3[1]<<") : "
               <<tmpresult);
      if (tmpresult>maxresult){
        maxresult = tmpresult;
        resultpoint[0]=p3[0]; resultpoint[1]=p3[1];
      }
      if (maxresult>mincorrelation) break;

      NCC(p1, p4, ida1, ida2, halfnccwinsize, tmpresult);
      BIASDOUT(D_REGION_MATCHER_SEARCH_EVEN,"("<<p4[0]<<", "<<p4[1]<<") : "
               <<tmpresult);
      if (tmpresult>maxresult){
        maxresult = tmpresult;
        resultpoint[0]=p4[0]; resultpoint[1]=p4[1];
      }
      if (maxresult>mincorrelation) break;
    } 
    if (maxresult<mincorrelation){
      p3[0]=p2[0]-sr+1;
      p4[0]=p2[0]+sr;
      for (p3[1]=p2[1]-sr+2, p4[1]=p2[1]-sr+2; p3[1]<=p2[1]+sr-1; 
           p3[1]++, p4[1]++){
        NCC(p1, p3, ida1, ida2, halfnccwinsize, tmpresult);
        BIASDOUT(D_REGION_MATCHER_SEARCH_EVEN,"("<<p3[0]<<", "<<p3[1]<<") : "
                 <<tmpresult);
        if (tmpresult>maxresult){
          maxresult = tmpresult;
          resultpoint[0]=p3[0]; resultpoint[1]=p3[1];
        }
        if (maxresult>mincorrelation) break;

        NCC(p1, p4, ida1, ida2, halfnccwinsize, tmpresult);
        BIASDOUT(D_REGION_MATCHER_SEARCH_EVEN,"("<<p4[0]<<", "<<p4[1]<<") : "
                 <<tmpresult);
        if (tmpresult>maxresult) {
          maxresult = tmpresult;
          resultpoint[0]=p4[0]; resultpoint[1]=p4[1];
        }
        if (maxresult>mincorrelation) break;
      } 
    }
    if (maxresult>mincorrelation){
      result = maxresult;
      BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "succeeded at searchradius "<<sr
               <<" with "<<result<<" correlation point: "<<resultpoint);
      break;
    }
  } // for (sr=1; sr<=halfsearchwinsize; sr++)
  result=maxresult;
  if (mincorrelation < 1.0 && result < mincorrelation){
    BIASDOUT(D_REGION_MATCHER_SEARCHNCC,"no success within "<<p2[0]<<" "
             <<p2[1]
             <<" +/- "<<halfsearchwinsize<<"  max correlation "<<result
             <<" at "<<resultpoint<< " still lower than "<<mincorrelation);
    res=-3;
  } else if (mincorrelation >= 1.0){
    BIASDOUT(D_REGION_MATCHER_SEARCHNCC,"searched within "<<p2[0]<<" "<<p2[1]
             <<" +/- "<<halfsearchwinsize<<" found max correlation "<<result
             <<" at "<<resultpoint[0]<<", "<<resultpoint[1]);
  }
  //  } // if (maxresult > result)

  return res;
}

template <class StorageType>
KLT_TYPE RegionMatcher::_Bilinear(StorageType **ida, KLT_TYPE x, KLT_TYPE y)
{
  unsigned int x_floor=(unsigned int)floor(x);
  unsigned int y_floor=(unsigned int)floor(y);
  unsigned int x_ceil=(unsigned int)ceil(x);
  unsigned int y_ceil=(unsigned int)ceil(y);
  StorageType ul = ida[y_floor][x_floor];
  StorageType ur = ida[y_floor][x_ceil];
  StorageType ll = ida[y_ceil][x_floor];
  StorageType lr = ida[y_ceil][x_ceil];
  KLT_TYPE dy = y - y_floor;
  KLT_TYPE dx = x - x_floor;
  
  return ul + dy*(ll - ul) + dx*(ur - ul) + dy*dx*(ul + lr - ll - ur );
}

KLT_TYPE RegionMatcher::_Bilinear(unsigned char**ida, 
                                                 KLT_TYPE x, KLT_TYPE y)
{
//   unsigned int x_floor=(unsigned int)floor(x);
//   unsigned int y_floor=(unsigned int)floor(y);
//   unsigned int x_ceil=(unsigned int)ceil(x);
//   unsigned int y_ceil=(unsigned int)ceil(y);
//   int ul = (int)(ida[y_floor][x_floor]);
//   int ur = (int)(ida[y_floor][x_ceil]);
//   int ll = (int)(ida[y_ceil][x_floor]);
//   int lr = (int)(ida[y_ceil][x_ceil]);
//   KLT_TYPE dy = y - y_floor;
//   KLT_TYPE dx = x - x_floor;
  
//   return ul + dy*(ll - ul) + dx*(ur - ul) + dy*dx*(ul + lr - ll - ur );
  register unsigned int x_floor = (unsigned int)floor(x);
  register unsigned int y_floor = (unsigned int)floor(y);
  register unsigned int x_ceil = x_floor+1;
  register unsigned int y_ceil = y_floor+1;
  register unsigned int ul = (unsigned int)ida[y_floor][x_floor];
  register unsigned int ur = (unsigned int)ida[y_floor][x_ceil];
  register unsigned int ll = (unsigned int)ida[y_ceil][x_floor];
  register unsigned int lr = (unsigned int)ida[y_ceil][x_ceil];
  register unsigned int dy = (unsigned int)rint((y - y_floor)*256);
  register unsigned int dx = (unsigned int)rint((x - x_floor)*256);

  return (unsigned char)(((ul<<16) + (dy<<8)*(ll - ul) + (dx<<8)*(ur - ul) + 
                          dy*dx*(ul - ll - ur + lr))>>16);
}

template <class StorageType>
void RegionMatcher::_BilinearRegion(StorageType **ida, KLT_TYPE x, 
                                    KLT_TYPE y, unsigned hws, 
                                    Matrix<KLT_TYPE>& res)
{
  unsigned  x_floor = (unsigned int)floor(x);
  unsigned y_floor = (unsigned int)floor(y);
  unsigned x_floor_end = x_floor+hws+1;
  unsigned y_floor_end = y_floor+hws+1;
  unsigned rf, rc, cf, cc, r, c;
  StorageType ul, ur, ll, lr;
  register KLT_TYPE dy = (KLT_TYPE)(y - y_floor);
  register KLT_TYPE dx = (KLT_TYPE)(x - x_floor);
  register KLT_TYPE dxy = dx*dy;
  for (rf=y_floor-hws, rc=rf+1, r=0; rf<y_floor_end; rf++, rc++, r++){
    for (cf=x_floor-hws, cc=cf+1, c=0; cf<x_floor_end; cf++, cc++, c++){
      ul = ida[rf][cf];
      ur = ida[rf][cc];
      ll = ida[rc][cf];
      lr = ida[rc][cc];
      res[r][c]=(KLT_TYPE)(ul + dy*(ll - ul) + dx*(ur - ul) + 
                           dxy*(ul + lr - ll - ur));
    }
  }
}


void RegionMatcher::_BilinearRegion(unsigned char **ida, KLT_TYPE x, 
				    KLT_TYPE y, unsigned hws, 
				    Matrix<KLT_TYPE>& res)
{
  unsigned int x_floor = (unsigned int)floor(x);
  unsigned int y_floor = (unsigned int)floor(y);
  unsigned int x_floor_end = x_floor+hws+1;
  unsigned int y_floor_end = y_floor+hws+1;
  unsigned rf, rc, cf, cc, r, c;
  unsigned ul, ur, ll, lr;
  register unsigned int dy = (unsigned int)rint((y - y_floor)*256);
  register unsigned int dx = (unsigned int)rint((x - x_floor)*256);
  register unsigned int dxy = dx * dy;
  for (rf=y_floor-hws, rc=rf+1, r=0; rf<y_floor_end; rf++, rc++, r++){
    for (cf=x_floor-hws, cc=cf+1, c=0; cf<x_floor_end; cf++, cc++, c++){
      ul = (unsigned)ida[rf][cf];
      ur = (unsigned)ida[rf][cc];
      ll = (unsigned)ida[rc][cf];
      lr = (unsigned)ida[rc][cc];
      res[r][c]=(unsigned char)(((ul<<16) + (dy<<8)*(ll - ul) + 
                                 (dx<<8)*(ur - ul) + 
                                 dxy*(ul - ll - ur + lr))>>16);
    }
  }
  //BIASERR("unfinished and untested");
}     

/*
template <class StorageType>
KLT_TYPE RegionMatcher::_BilinearGrad(StorageType **ida, KLT_TYPE x, KLT_TYPE y)
{
  unsigned int x_floor=(unsigned int)floor(x);
  unsigned int y_floor=(unsigned int)floor(y);
  unsigned int x_ceil=(unsigned int)ceil(x);
  unsigned int y_ceil=(unsigned int)ceil(y);
  KLT_TYPE ul = (KLT_TYPE)(ida[y_floor][x_floor])-SOBEL_GRAD_UC_ZERO;
  KLT_TYPE ur = (KLT_TYPE)(ida[y_floor][x_ceil])-SOBEL_GRAD_UC_ZERO;
  KLT_TYPE ll = (KLT_TYPE)(ida[y_ceil][x_floor])-SOBEL_GRAD_UC_ZERO;
  KLT_TYPE lr = (KLT_TYPE)(ida[y_ceil][x_ceil])-SOBEL_GRAD_UC_ZERO;
  KLT_TYPE dy = y - y_floor;
  KLT_TYPE dx = x - x_floor;
  
  return ul + dy*(ll - ul) + dx*(ur - ul) + dy*dx*(ul + lr - ll - ur );
}

// also moves gradient in [-127, 127] befor interpolating 

template <>
KLT_TYPE RegionMatcher::_BilinearGrad<unsigned char>(unsigned char **ida, KLT_TYPE x, KLT_TYPE y)
{
  unsigned int x_floor=(unsigned int)floor(x);
  unsigned int y_floor=(unsigned int)floor(y);
  unsigned int x_ceil=(unsigned int)ceil(x);
  unsigned int y_ceil=(unsigned int)ceil(y);
  int ul = (int)(ida[y_floor][x_floor])-SOBEL_GRAD_UC_ZERO;
  int ur = (int)(ida[y_floor][x_ceil])-SOBEL_GRAD_UC_ZERO;
  int ll = (int)(ida[y_ceil][x_floor])-SOBEL_GRAD_UC_ZERO;
  int lr = (int)(ida[y_ceil][x_ceil])-SOBEL_GRAD_UC_ZERO;
  KLT_TYPE dy = y - y_floor;
  KLT_TYPE dx = x - x_floor;
  
  return ul + dy*(ll - ul) + dx*(ur - ul) + dy*dx*(ul + lr - ll - ur );
}

template <class StorageType>
void RegionMatcher::_BilinearRegionGrad(StorageType **ida, KLT_TYPE x, 
                                        KLT_TYPE y, unsigned hws, 
                                        Matrix<KLT_TYPE>& res)
{
  unsigned x_floor = (unsigned int)floor(x);
  unsigned y_floor = (unsigned int)floor(y);
  unsigned x_floor_end = x_floor+hws+1;
  unsigned y_floor_end = y_floor+hws+1;
  unsigned rf, rc, cf, cc, r, c;
  KLT_TYPE ul, ur, ll, lr;
  register KLT_TYPE dy = (KLT_TYPE)(y - y_floor);
  register KLT_TYPE dx = (KLT_TYPE)(x - x_floor);
  register KLT_TYPE dxy = dx*dy;
  for (rf=y_floor-hws, rc=rf+1, r=0; rf<y_floor_end; rf++, rc++, r++){
    for (cf=x_floor-hws, cc=cf+1, c=0; cf<x_floor_end; cf++, cc++, c++){
      ul = (KLT_TYPE)ida[rf][cf]-(KLT_TYPE)SOBEL_GRAD_UC_ZERO;
      ur = (KLT_TYPE)ida[rf][cc]-(KLT_TYPE)SOBEL_GRAD_UC_ZERO;
      ll = (KLT_TYPE)ida[rc][cf]-(KLT_TYPE)SOBEL_GRAD_UC_ZERO;
      lr = (KLT_TYPE)ida[rc][cc]-(KLT_TYPE)SOBEL_GRAD_UC_ZERO;
      res[r][c]=(ul + dy*(ll - ul) + dx*(ur - ul) + dxy*(ul + lr - ll - ur));
    }
  }

}
*/

template <class StorageType>
int RegionMatcher::ParabolaNCC(unsigned int p1[2], unsigned int p2[2],
                               StorageType **ida1, StorageType **ida2,
                               unsigned int halfwinsize, KLT_TYPE result[2])
{
  int res=0;

  double x1, x2, x3, y1, y2, y3;
  unsigned int p2tmp[2];
  BIASDOUT(D_REGION_MATCHER_PARABOLA, "p1 ("<<p1[0]<<", "<<p1[1]<<")\tp2 ("
           <<p2[0]<<", "<<p2[1]<<")\tida1: "<<&ida1[0]<<" ida2: "<<&ida2[0]
           <<"  halfwinsize: "<<halfwinsize);
  NCC(p1, p2, ida1, ida2, halfwinsize, x2);
  //cerr <<"x2: "<<x2<<endl;
  y2=x2;
  p2tmp[0]=p2[0];
  p2tmp[1]=p2[1]-1;
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, y1);
  p2tmp[1]=p2[1]+1;
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, y3);
  p2tmp[0]=p2[0]-1;
  p2tmp[1]=p2[1];
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, x1);
  p2tmp[0]=p2[0]+1;
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, x3);


  if (x3>=x2 || x1>=x2) { 
    BIASERR("invalid correlation data "<<x1<<" "<<x2
            <<" "<<x3<<"\n\tcorrelation between ("
            <<p1[0]<<", "<<p1[1]<<") and ("<<p2[0]
            <<", "<<p2[1]<<") is not a local maximum"); 
    return -1;
  }
  if (y3>=y2 || y1>=y2) { 
    BIASERR("invalid correlation data "<<y1<<" "<<y2
            <<" "<<y3<<"\n\tcorrelation between ("
            <<p1[0]<<", "<<p1[1]<<") and ("<<p2[0]
            <<", "<<p2[1]<<") is not a local maximum"); 
  return -1;
  }

  result[0]=p2[0]+_ParabolaApprox(x1, x2, x3);
  result[1]=p2[1]+_ParabolaApprox(y1, y2, y3);
  BIASDOUT(D_REGION_MATCHER_PARABOLA, "\n(x1 x2 x3) : res\t("<<x1<<" "<<x2<<" "
           <<x3<<") : "<<result[0]-p2[0]<<"\n(y1 y2 y3) : res\t("
           <<y1<<" "<<y2<<" "<<y3<<") : "<<result[1]-p2[1]);
  return res;
}


template <class StorageType>
int RegionMatcher::ParabolaNCC5(unsigned int p1[2], unsigned int p2[2],
                                StorageType **ida1, StorageType **ida2,
                                unsigned int halfwinsize, KLT_TYPE result[2])
{
  int res=0;

  double x1, x2, x3, x4, x5, y1, y2, y3, y4, y5;
  unsigned int p2tmp[2];
  BIASDOUT(D_REGION_MATCHER_PARABOLA, "p1 ("<<p1[0]<<", "<<p1[1]<<")\tp2 ("
           <<p2[0]<<", "<<p2[1]<<")\tida1: "<<&ida1[0]<<" ida2: "<<&ida2[0]
           <<"  halfwinsize: "<<halfwinsize);
  NCC(p1, p2, ida1, ida2, halfwinsize, x3);
  //cerr <<"x2: "<<x2<<endl;
  y3=x3;
  p2tmp[0]=p2[0];
  p2tmp[1]=p2[1]-1;
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, y2);
  p2tmp[1]=p2[1]-2;
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, y1);
  p2tmp[1]=p2[1]+1;
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, y4);
  p2tmp[1]=p2[1]+2;
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, y5);
  p2tmp[0]=p2[0]-1;
  p2tmp[1]=p2[1];
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, x2);
  p2tmp[0]=p2[0]-2;
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, x1);
  p2tmp[0]=p2[0]+1;
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, x4);
  p2tmp[0]=p2[0]+2;
  NCC(p1, p2tmp, ida1, ida2, halfwinsize, x5);


  if (x1>=x3 || x2>=x3 || x4>=x3 || x5>=x3 ) { 
    BIASERR("invalid correlation data "<<x1<<" "<<x2
            <<" "<<x3<<" "<<x4<<" "<<x5<<"\n\tcorrelation between ("
            <<p1[0]<<", "<<p1[1]<<") and ("<<p2[0]
            <<", "<<p2[1]<<") is not a local maximum"); 
    return -1;
  }
  if (y1>=y3 || y2>=y3 || y4>=y3 || y5>=y3 ) { 
    BIASERR("invalid correlation data "<<y1<<" "<<y2
            <<" "<<y3<<" "<<y4<<" "<<y5<<"\n\tcorrelation between ("
            <<p1[0]<<", "<<p1[1]<<") and ("<<p2[0]
            <<", "<<p2[1]<<") is not a local maximum"); 
    return -1;
  }

  result[0]=p2[0]+_ParabolaApprox(x1, x2, x3, x4, x5);
  result[1]=p2[1]+_ParabolaApprox(y1, y2, y3, y4, y5);
  BIASDOUT(D_REGION_MATCHER_PARABOLA, 
           "\n(x1 x2 x3 x4 x5) : res\t("<<x1<<" "<<x2<<" "<<x3<<" "<<x4
           <<" "<<x5<<") : "<<result[0]-p2[0]<<"\n(y1 y2 y3 y4 y5) : res\t("
           <<y1<<" "<<y2<<" "<<y3<<" "<<y4<<" "<<y5<<") : "<<result[1]-p2[1]);
  return res;
}
