/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _TrackerSynth_hh_
#define _TrackerSynth_hh_

#include <bias_config.h>

#include <Matcher2D/Tracker.hh>

#include <vector>
#include <string>

#define D_TS_READ    0x80000000
#define D_TS_REPLACE 0x40000000

namespace BIAS{

  /** @class TrackerSynth
      @brief returns ground truth tracks read from file, for debugging
      @author woelk 01/2006 */
  template <class StorageType, class CalculationType>
  class BIASMatcher2D_EXPORT TrackerSynth 
    : public Tracker<StorageType, CalculationType>
  {
  public:
    TrackerSynth();
    virtual ~TrackerSynth();

    /** Tracks all points from the internal data structure Points_ and 
        afterwards replaces them by the nearest track read from file
        @author woelk 01/2006 */
    virtual int Track(PyramidImage<CalculationType>& pim, 
                      PyramidImage<CalculationType>& gradx,
                      PyramidImage<CalculationType>& grady);
    
    /** Tracks all points in Points_ using prediction as previous knowledge
        and afterwards replaces them by the nearest track read from file.
        @author woelk 01/2006 */
    virtual int Track(PyramidImage<CalculationType>& pim, 
                      PyramidImage<CalculationType>& gradx,
                      PyramidImage<CalculationType>& grady,
                      const std::vector<HomgPoint2D>& prediction,
                      const std::vector<Matrix2x2<KLT_TYPE> >&affineprediction=
                      std::vector<Matrix2x2<KLT_TYPE> >());

    using Tracker<StorageType, CalculationType>::Points_;
    using Tracker<StorageType, CalculationType>::PyIndex_;

  protected:
    //BIAS_EXPORT_STL_VECTOR( BIASMatcher2D_EXPORT, BIAS::HomgPoint2D )

    /// tracks read from file by ReadTracks_
    std::vector<std::vector<BIAS::HomgPoint2D> > Tracks_;

    /// current frame number
    int Frame_;

    /** reads tracks from file and stores them in Tracks_ */
    void ReadTracks_(const std::string& filename);

    /** finds the nearest point to p in all frames in Tracks_.
        Returns nearest_frame_index and nearest_track_index */
    void FindNearest_(HomgPoint2D& p, int &nearest_frame_index, 
                      int &nearest_track_index);

    /** Finds the nearest point to p in Tracks_[frame]. Returns the 
        track index, i.e. the nearest point to p i Tracks_[frame][track index] 
    */
    int FindNearest_(HomgPoint2D& p, const int frame);

    /** for all succesfully tracked points p in Points_[*PyIndex]
        it calls FindNearest_(p, Frame_) and replaces 
        Points_[*PyIndex] by the so found track. 
        Adds Gaussian Noise and outlier to the replaced points. */
    void Track_();

    /** adds spatial gaussian noise to Points_[*PyIndex_] */
    void AddGaussianNoise_();

    /** adds outliers to Points_[*PyIndex_] */
    void AddOutlier_();

  };
}

#endif // _TrackerSynth_hh_
