/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _UncertaintyTransformBase_hh_
#define _UncertaintyTransformBase_hh_

#include <Base/Debug/Debug.hh>

namespace BIAS {

  // forward declaration
  template <class T> class BIASMathBase_EXPORT Matrix;
  template <class T> class BIASMathBase_EXPORT Vector;

  /** @class UncertaintyTransformBase
      @brief base class for all uncertainty transforms 

      @author woelk 11/2006 */
  class BIASMathAlgo_EXPORT UncertaintyTransformBase 
    : public Debug
  {
  public:
    virtual ~UncertaintyTransformBase() {};

    /** @brief computes the second order approximation of the 
        transformations of the mean and the associated covariance for the 
        point transformation Transform_().
        @returns 0 on success 
                -1 when transformation of mean failed
                +1 when transfromation of sigma point failed */ 
    virtual int Transform(const Vector<double>& src_mean, 
                          const Matrix<double>& src_cov,
                          Vector<double>& dst_mean, 
                          Matrix<double>& dst_cov) const = 0;

  protected:
    UncertaintyTransformBase() : Debug() {};
   
  }; // class

} // namespace

#endif // _UncertaintyTransformBase_hh_
