/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __BIASCScommServer__
#define __BIASCScommServer__

#include "CScommBase.hh"

//#include <Base/Common/BIASpragmaStart.hh>

namespace BIAS {
  /** @class CScommServer
  @brief class for sending/receiving data between clients and servers

  Only registered msgs will be accepted at the server side,
  similar the client will accept only registered msgs from the server too.
  The server waits on a DEFUALT_PORT for connections from clients,
  which need to know the hostname of the server. 
  Usage: <br>

  -# Konstruktor: CScommServer Server;

  -# Register msgs:  Server.RegisterMsg(msgName, BIAS::CS_FLOAT, 3);

  -# Enable receiver:  Server.WaitForConnections();

  -# send and/or get Data:  Server.SendMsg(msgName,vecOfFloats);<br>
  Server.GetData(msgName, newVecOffloats);

  See the examples also.
  @author Daniel Grest, Jan. 2004
  */
  class BIASNetworkComm_EXPORT CScommServer : public CScommBase {
  public:

    /** Set threaded to false for deactivating the receive-thread. Data will be
    read from TCP-buffer only when GetData methods are called. Threaded
    receive is activated by default!
    Beware: if there's no msg in tcp-buffer, GetData will BLOCK until there
    is any new msg (this need not be the one GetData was called for,
    for example binary instead of int)
    if receive-buffer is completly filled, send blocks until msg
    can be delivered!
    */
    CScommServer(bool threaded = true);
    ~CScommServer();

    /** Init function, returns instantly and enables the server to accept
    connections on the given port
    */
    void WaitForConnections(unsigned int port=D_CS_DEFAULT_PORT); 

  protected: 
    void CreateListenThread_();
    bool terminateListener_, listenThreadActive_;
    pthread_t pthreadListener_;

    friend void* listenthread(void *data);  
  };

  /// these are wrapper to comply to the pthread_create() interface
  void *listenthread(void *data);

}

//#include <Base/Common/BIASpragmaEnd.hh>

#endif
