#ifndef __ShowCamWxThreads_hh__
#define __ShowCamWxThreads_hh__

#include <bias_config.h>

#include <Gui/StringConv.hh>
#include <Image/Camera.hh>
#include <Base/Image/ImageIO.hh>

#include <wx/wx.h>

/**
   @class  ImageSaveThread
   @brief  This class creates a thread which stores an image in a file
           if rescale is not zero then the stored image is downsampled
           by this factor.
   @author Christoffer Menk
*/
namespace BIAS {

  class ImageSaveThread : public wxThread
  {
  public:
    ImageSaveThread(const wxString& filename,
                    const BIAS::Camera<unsigned char>& img,
                    const int& downsample)
      : wxThread(wxTHREAD_DETACHED)  
    {
      DownSample_ = downsample;
      Img_ = img;
      Filename_ = filename;
    }   
    ~ImageSaveThread() {}
 
    virtual ExitCode Entry()
    {
      if (DownSample_ != 0)
      {
        // rescale image and store it to disk
        Camera<unsigned char> rescaledImage;        
        Rescale<unsigned char, unsigned char> rescale;
        rescale.Downsample(Img_, rescaledImage,
                           (int)(Img_.GetWidth() / DownSample_),
                           (int)(Img_.GetHeight() / DownSample_));
        BIAS::ImageIO::Write(WxToAscii(Filename_), rescaledImage);        
      }
      else
      {
        // save image without rescaling
        BIAS::ImageIO::Write(WxToAscii(Filename_), Img_);
      }
      return NULL;
    }
  
  private:
    wxString Filename_;
    BIAS::Camera<unsigned char> Img_;
    int DownSample_;
  };

/**
   @class  GrabSingleThread
   @brief  This class creates a thread which grabs a single image
   @author Christoffer Menk
*/  
  class GrabSingleThread : public wxThread
  {
  public:
    GrabSingleThread(BIAS::VideoSource* cam,
                     BIAS::Camera<unsigned char>* img)
      : wxThread(wxTHREAD_JOINABLE)  
    {
      Img_ = img;
      Cam_ = cam;
    }   
    ~GrabSingleThread() {}
 
    virtual ExitCode Entry()
    {
      Cam_->GrabSingle(*Img_);
      return NULL;
    }
    
    virtual void OnExit() {
    }
    
  private:
    BIAS::VideoSource* Cam_;
    BIAS::Camera<unsigned char>* Img_; 
  };

} // end of namespace BIAS
  
#endif // __ShowCamWxThreads_hh__
