/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BIASpragma.hh>

#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>
#include <stdio.h>


using namespace BIAS;
using namespace std;

void usage()
{
  cout << "Usage: biassetprojection imagefile projectionfile [destfile]" << endl;
  cout << "This program opens the image file and the projection file" << endl
       << "given by the command line, copies the projection into the image" << endl
       << "and saves the image to destfile. If no destfile is given," << endl
       << "the source image file is overwritten." << endl;
}

/** @class biassetprojection
    @ingroup g_tools
    @brief load image and projection given by the command line,
    set projection in image and save image to disk.
    @author jkollmann */
int main (int argc, char* argv[])
{
  if (argc < 3)
  {
    usage();
    return 0;
  }

  // load image
  Camera<unsigned char> image;
  if (ImageIO::Load(argv[1], image) != 0)
  {
    BIASERR("Failed to load image: " << argv[1]);
    return 1;
  }

  // load projection
  Projection projection;
#ifdef BIAS_HAVE_XML2
  if (projection.XMLRead(argv[2]) != 0)
  {
    BIASERR("Failed to load projection: " << argv[2]);
    return 1;
  }
#endif //BIAS_HAVE_XML2

  // set projection in image
  image.SetProj(projection);
  image.UpdateMetaData();

  // choose destination file
  string destfile = argv[1];
  if (argc >= 4)
  {
    destfile = argv[3];
  }

  // save file
  if (ImageIO::Save(destfile, image, ImageIO::FF_auto, BIAS_DEFAULT_SYNC,
                    BIAS_DEFAULT_IMAGE_QUALITY, false, true) != 0)
  {
    BIASERR("Failed to save image: " << destfile);
    return 1;
  }
  
  return 0;  
}
