/**
\todo implement command line switch to terminate biasviewwx automatically 
with useful return code for dart ADD_TEST to be able to test biasvviewwx 
run in nightly cmake Dart dashboard (JW)
FIXME: 
- float canvcas needs scrollwbars/scrollwindow for width as it width is bound 
to the width of the panel with the scaler buttons and sliders below. (Linux only problem)
(amattal, jw)
- for float ScaledImageCanvas: the settings for scale, offset are lost when an Uchar image is loaded.
+ RemoveAll/RemoveCurrent are buggy.
*/

#include <wx/wx.h>
#include <wx/spinctrl.h>
#include <wx/dnd.h>
#include <wx/filename.h>
#include <wx/arrstr.h>


#include <string>
#include <sstream>
#include <limits>

#include <Gui/ImageCanvas.hh>
#include <Base/Image/ImageConvert.hh>
#include <Gui/ConsoleRedirectIO.hh>
#include <Gui/StringConv.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Common/FileHandling.hh>
#include <Image/Camera.hh>
#include <Utils/Param.hh>


#define FRAME_DEFAULT_SIZE  wxSize(720,500)

using namespace std;
using namespace BIAS;

#undef WXWIN_COMPATIBILITY


#if !wxUSE_DRAG_AND_DROP
#  error The dndFileTarget class requires drag and drop support in the WX library. Please recompile wxWidgets. __FILE__: __LINE__
#endif

/*
===============================================================================
MainFrame
===============================================================================
*/

/// menu item IDs
enum biasviewxx_IDS
{

  // standard ids
  ID_About= wxID_ABOUT,
  ID_Exit = wxID_EXIT,

  // our custom ids
  ID_Load = wxID_HIGHEST+2001, /* first custom id. */

  ID_RemoveCurrent,
  ID_RemoveAllButCurrent,

  ID_Info,
  ID_MetaData,
  ID_KeyInfo,

  // toolbar item IDs
  ID_Prev,
  ID_Next,
  ID_RunStop,
  ID_Histogramm,
  ID_Delay,
  ID_Fit,

  // timer ID
  ID_Timer,

  ID_ChangeChannel_1,
  ID_ChangeChannel_2,
  ID_ChangeChannel_3,
  ID_ChangeChannel_4,
  ID_ChangeChannel_5,
  ID_ChangeChannel_6,
  ID_ChangeChannel_7,
  ID_ChangeChannel_8,
  ID_ChangeChannel_ALL

  ,ID_SaveList,
  ID_ImageValues

};

class MainFrame : public wxFrame
{

public:
  friend class DragAndDropTarget;

  MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size);

  void SetFileList(const vector<string>& fileNames);
  void LoadImages(const wxArrayString &paths);

  /// fits the frame size if it is too small for curent image
  void FitSizetIfTooSmall();

  /// workaround adapting client canvases size
  void RefitClients();

  /// some compressed images display size is bigger then original size
  /// e.b. X3B images can be displayed three times a swide as loaded
  /// Uncompress after laod to keep same display and orig image size 
  /// which eases coordinate transforms
  /// between mouse, display rgb and orig image. (JW)
  void UncompressImage(BIAS::ImageBase & image);

  /// print info such as dim, video format ro info stream. 
  /// Used by GetInfo (JW)
  void AddInfo(const BIAS::ImageBase & image, std::stringstream & info);


protected:
  void LoadImageFromList(int index);

  //SizerWindow and its content, _MainSizer which handles
  //the sizing-events for the canvases
  wxWindow*           _SizerWindow;
  wxBoxSizer*         _MainSizer;

  //Canvases for UChar and Float images
  ImageCanvas*        _ImageCanvasUC;
  ScaledImageCanvas*  _ImageCanvasFloat;
  wxPoint             _HistoPosition;
  wxSize              _HistoSize;

  vector<string>      _FileNames;
  int                 _CurrentImage;
  string              _MetaData;
  wxTimer             _Timer;
  bool                _BlockAnimation;
  wxSpinCtrl*         _DelaySpin;
  wxCheckBox*         _FitCheckBox;
  wxCheckBox*         _ReverseCheckBox;
  wxButton*           _RunStopButton;
  wxButton*           _HistogrammButton;
  bool                _NeedReload;

  bool                _ShowHistogramm;

  unsigned int        _SelectedChannel;

  // for file open dialog
  wxString            _DefaultFile;
  wxString            _DefaultDir;

  // event handlers
  void OnLoad(wxCommandEvent& event);
  void OnRemoveCurrent(wxCommandEvent& event);
  void OnRemoveAllButCurrent(wxCommandEvent& event);
  void OnInfo(wxCommandEvent& event);
  void OnMetaData(wxCommandEvent& event);
  void OnExit(wxCommandEvent& event);
  void OnKeyInfo(wxCommandEvent& event);
  void OnAbout(wxCommandEvent& event);
  void OnImageValues(wxCommandEvent& event);


  /// save all filenames to disk
  void OnSaveList(wxCommandEvent& event);

  void OnNext(wxCommandEvent& event);
  void OnPrev(wxCommandEvent& event);
  void OnRunStop(wxCommandEvent& event);

  void OnHistogramm(wxCommandEvent& event);
  void ShowHistogramm(ImageBase& image);

  void OnDelay(wxSpinEvent& event);
  void OnFit(wxCommandEvent& event);

  void OnChangeChannel_1(wxCommandEvent& event);
  void OnChangeChannel_2(wxCommandEvent& event);
  void OnChangeChannel_3(wxCommandEvent& event);
  void OnChangeChannel_4(wxCommandEvent& event);
  void OnChangeChannel_5(wxCommandEvent& event);
  void OnChangeChannel_6(wxCommandEvent& event);
  void OnChangeChannel_7(wxCommandEvent& event);
  void OnChangeChannel_8(wxCommandEvent& event);
  void OnChangeChannel_ALL(wxCommandEvent& event);

  void OnTimer(wxTimerEvent& event);
  void OnIdle(wxIdleEvent& event);
  void OnSize(wxSizeEvent& event);

  std::string infoRaw_;//< global info field filled by Uncompress

  DECLARE_EVENT_TABLE()
};


BEGIN_EVENT_TABLE(MainFrame, wxFrame)
EVT_MENU    (ID_Exit,       MainFrame::OnExit)
EVT_MENU    (ID_Load,       MainFrame::OnLoad)
EVT_MENU    (ID_RemoveCurrent,MainFrame::OnRemoveCurrent)
EVT_MENU    (ID_RemoveAllButCurrent, MainFrame::OnRemoveAllButCurrent)
EVT_MENU    (ID_Info,       MainFrame::OnInfo)
EVT_MENU    (ID_MetaData,   MainFrame::OnMetaData)
EVT_MENU    (ID_SaveList,   MainFrame::OnSaveList)
EVT_MENU    (ID_ImageValues,   MainFrame::OnImageValues)

EVT_MENU    (ID_KeyInfo,    MainFrame::OnKeyInfo)
EVT_MENU    (ID_About,      MainFrame::OnAbout)

EVT_BUTTON  (ID_Next,       MainFrame::OnNext)
EVT_BUTTON  (ID_Prev,       MainFrame::OnPrev)
EVT_BUTTON  (ID_RunStop,    MainFrame::OnRunStop)
EVT_BUTTON  (ID_Histogramm, MainFrame::OnHistogramm)
EVT_SPINCTRL(ID_Delay,      MainFrame::OnDelay)
EVT_CHECKBOX(ID_Fit,        MainFrame::OnFit)

EVT_MENU    (ID_Next,       MainFrame::OnNext)
EVT_MENU    (ID_Prev,       MainFrame::OnPrev)
EVT_MENU    (ID_RunStop,    MainFrame::OnRunStop)

EVT_MENU    (ID_ChangeChannel_1,    MainFrame::OnChangeChannel_1)
EVT_MENU    (ID_ChangeChannel_2,    MainFrame::OnChangeChannel_2)
EVT_MENU    (ID_ChangeChannel_3,    MainFrame::OnChangeChannel_3)
EVT_MENU    (ID_ChangeChannel_4,    MainFrame::OnChangeChannel_4)
EVT_MENU    (ID_ChangeChannel_5,    MainFrame::OnChangeChannel_5)
EVT_MENU    (ID_ChangeChannel_6,    MainFrame::OnChangeChannel_6)
EVT_MENU    (ID_ChangeChannel_7,    MainFrame::OnChangeChannel_7)
EVT_MENU    (ID_ChangeChannel_8,    MainFrame::OnChangeChannel_8)
EVT_MENU    (ID_ChangeChannel_ALL,  MainFrame::OnChangeChannel_ALL)

EVT_TIMER   (ID_Timer, MainFrame::OnTimer)
EVT_IDLE    (MainFrame::OnIdle)
EVT_SIZE    (MainFrame::OnSize)
END_EVENT_TABLE()

// icon
#if !defined(__WXMSW__) && !defined(__WXPM__)
#  include "biasviewwx.xpm"
#endif


// ctor 
MainFrame::MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
: wxFrame((wxFrame*)NULL, -1, title, pos, size)
,_ImageCanvasUC(NULL)
,_ImageCanvasFloat(NULL)
,_FitCheckBox(NULL)
{
  //#ifdef WIN32 // until we include an appropriate  XPM
  SetIcon(wxICON(biasviewwx));
  //#endif

  _CurrentImage     = 0;
  _DefaultFile      = wxT("ucim.pgm");
  _DefaultDir       = wxT("");
  _BlockAnimation   = true;
  _SelectedChannel  = 0;

  _ShowHistogramm = false;
  _HistoPosition = wxPoint(10,10);
  _HistoSize = wxSize(100,100);

  // init timer
  _Timer.SetOwner(this, ID_Timer);

  // create file menu
  wxMenu* fileMenu = new wxMenu;
  fileMenu->Append(ID_Load, wxT("&Open/Load...\tCtrl-O"), wxT("Open a file from disk for loading")); 
  fileMenu->Append(ID_RemoveCurrent, 
    wxT("&Remove current image...\tCtrl-R"), wxT("Remove current file from viewer list"));
  fileMenu->Append(ID_RemoveAllButCurrent, 
    wxT("Remove &all but current image"), wxT("Remove all files from viewer list "));
  fileMenu->AppendSeparator();
  fileMenu->Append(ID_Info, wxT("Image Info...\tCtrl-I"));
  fileMenu->Append(ID_MetaData, wxT("Meta Data...\tCtrl-M"));
  fileMenu->Append(ID_ImageValues, wxT("Image Values..."));
  fileMenu->AppendSeparator();  
  fileMenu->Append(ID_SaveList, 
    wxT("&Save image list to .lst file ...\tCtrl-S"), wxT("save all list of all image filenames to .lst disk file."));
  fileMenu->AppendSeparator();
  fileMenu->Append(ID_Exit, wxT("E&xit"), wxT("Exit this application"));


  // create help menu
  wxMenu* helpMenu = new wxMenu;
  helpMenu->Append(ID_KeyInfo, wxT("&Keys...\tF1"));
  helpMenu->Append(ID_About, wxT("&About...\tCtrl-F1"));

  // create channel selection menu
  wxMenu* channelMenu = new wxMenu;
  channelMenu->Append(ID_ChangeChannel_ALL, wxT("&0 All Channels\tShift-F9"));
  channelMenu->AppendSeparator();
  channelMenu->Append(ID_ChangeChannel_1, wxT("&1 channel (red?)\tShift-F1"));
  channelMenu->Append(ID_ChangeChannel_2, wxT("&2 channel (green?)\tShift-F2"));
  channelMenu->Append(ID_ChangeChannel_3, wxT("&3 channel (blue?)\tShift-F3"));
  channelMenu->Append(ID_ChangeChannel_4, wxT("&4 channel (alpha?)\tShift-F4"));
  channelMenu->Append(ID_ChangeChannel_5, wxT("&5 channel\tShift-F5"));
  channelMenu->Append(ID_ChangeChannel_6, wxT("&6 channel\tShift-F6"));
  channelMenu->Append(ID_ChangeChannel_7, wxT("&7 channel\tShift-F7"));
  channelMenu->Append(ID_ChangeChannel_8, wxT("&8 channel\tShift-F8"));


  // create menu bar
  wxMenuBar* menuBar = new wxMenuBar;
  menuBar->Append(fileMenu, wxT("&File"));
  menuBar->Append(channelMenu, wxT("view &channel"));
  menuBar->Append(helpMenu, wxT("&Help"));
  SetMenuBar(menuBar);

  // create status bar
  const int numFields = 2;
  CreateStatusBar(numFields);
  int widths[] = { -2, 250 }; // sizes of fields in array, negative means relative *variable* size JW
  // positive means absolute *fixed* size FRISO
  SetStatusWidths(numFields, widths);
  SetStatusText(wxT("Welcome to BiasViewWX"), 0);

  // create tool bar
  wxToolBar* toolBar = new wxToolBar(this, -1);
  //wxToolBar* toolBar = CreateToolBar();
  toolBar->AddControl(new wxButton(toolBar, ID_Prev, 
                                   wxT(" [DOWN] Previous ")));
  wxButton* button_new = new wxButton(toolBar, ID_Next, 
                                      wxT(" Next [UP] "));
  toolBar->AddControl(button_new);
  _RunStopButton = new wxButton(toolBar, ID_RunStop, 
                                wxT(" Run [Space] "));
  toolBar->AddControl(_RunStopButton);
  _HistogrammButton = new wxButton(toolBar, ID_Histogramm, 
                                wxT(" Histogramm "));
  toolBar->AddControl(_HistogrammButton);
  
  _ReverseCheckBox = new wxCheckBox(toolBar, -1, wxT("Reverse"));
  toolBar->AddControl(_ReverseCheckBox);
  toolBar->AddSeparator();

  _DelaySpin = new wxSpinCtrl(toolBar, ID_Delay, wxT("100"),
    wxDefaultPosition, wxDefaultSize,
    wxSP_ARROW_KEYS, 1, 2000);
  toolBar->AddControl(_DelaySpin);

  toolBar->AddSeparator();
  _FitCheckBox = new wxCheckBox(toolBar, ID_Fit, wxT("Fit to window"));
  toolBar->AddControl(_FitCheckBox);
  toolBar->Realize();
  SetToolBar(toolBar);

  /************************************************
   * the format of wxwindow layout is as follows:
   *
   *  /------------------------------------\
   *  |              TOOLBAR               |
   *  |------------------------------------|
   *  |               SIZER                | 
   *  |------------------------------------|
   *  |             STATUSBAR              |
   *  \------------------------------------/
   *
   * so what we do - we add a _SizerWindow to our frame
   * (the frame handles resizing and refitting of a single
   * window AUTOMATICALLY) and then we add wxBoxSizer 
   * _MainSizer which contains both canvases canvases as 
   * children (so that the sizer decides what size should 
   * the canvases be of)
   ***********************************************/
  _MainSizer = new wxBoxSizer(wxVERTICAL);
  _SizerWindow = new wxWindow(this,-1);

  _SizerWindow->SetSizer(_MainSizer);

  // set the shortcuts
  std::vector< wxAcceleratorEntry > entries;

  //Some key-events were changed so that default wx-controls for
  //input-output would work (like ENTER or BACKSPACE, for instance)

  //Commented entries were at the beginning
  //   entries.push_back( wxAcceleratorEntry(wxACCEL_NORMAL, 
  //                                         WXK_RETURN, ID_Next));
  //   entries.push_back( wxAcceleratorEntry(wxACCEL_NORMAL, 
  //                                         WXK_BACK,   ID_Prev));
  entries.push_back( wxAcceleratorEntry(wxACCEL_NORMAL, 
                                        WXK_UP, ID_Next));
  entries.push_back( wxAcceleratorEntry(wxACCEL_NORMAL, 
                                        WXK_DOWN,   ID_Prev));
  //Commented entries were at the beginning
  //   entries.push_back( wxAcceleratorEntry(wxACCEL_CTRL,   
  //                                         WXK_SPACE,  ID_RunStop));
  entries.push_back( wxAcceleratorEntry(wxACCEL_NORMAL,   
                                        WXK_SPACE,  ID_RunStop));
  entries.push_back( wxAcceleratorEntry(wxACCEL_NORMAL,
                                        104, ID_Histogramm));

  entries.push_back( wxAcceleratorEntry(wxACCEL_SHIFT,  WXK_F1,     
                                        ID_ChangeChannel_1));
  entries.push_back( wxAcceleratorEntry(wxACCEL_SHIFT,  WXK_F2,     
                                        ID_ChangeChannel_2));
  entries.push_back( wxAcceleratorEntry(wxACCEL_SHIFT,  WXK_F3,     
                                        ID_ChangeChannel_3));
  entries.push_back( wxAcceleratorEntry(wxACCEL_SHIFT,  WXK_F4,     
                                        ID_ChangeChannel_4));
  entries.push_back( wxAcceleratorEntry(wxACCEL_SHIFT,  WXK_F5,  
                                        ID_ChangeChannel_5));
  entries.push_back( wxAcceleratorEntry(wxACCEL_SHIFT,  WXK_F6,
                                        ID_ChangeChannel_6));
  entries.push_back( wxAcceleratorEntry(wxACCEL_SHIFT,  WXK_F7,
                                        ID_ChangeChannel_7));
  entries.push_back( wxAcceleratorEntry(wxACCEL_SHIFT,  WXK_F8,
                                        ID_ChangeChannel_8));
  entries.push_back( wxAcceleratorEntry(wxACCEL_SHIFT,  WXK_F9,
                                        ID_ChangeChannel_ALL));

  BIASASSERT(entries.size()>0);
  wxAcceleratorTable accel(entries.size(), (wxAcceleratorEntry*)&entries[0] );
  SetAcceleratorTable(accel);

  button_new->SetFocus();
}

void MainFrame::SetFileList(const vector<string>& fileNames)
{
  _FileNames = fileNames;
  if (_FileNames.size() > 0)
  {
    LoadImageFromList(0);
  }
}

void MainFrame::UncompressImage(BIAS::ImageBase & image){
  if (image.GetColorModel()==BIAS::ImageBase::CM_PGR_XB3_F7M3_GBRG){
    // store info about raw image which will be lost by conversion
    stringstream ss;
    AddInfo(image, ss);
    infoRaw_ = ss.str();

    // uncompress raw to different size display image
    // the uncompression scheme can be selecetd via menu mode 
    // ... or hard coded for now
    BIAS::ImageBase tmp = image;
    ImageConvert::DeinterleaveHorizontal(tmp, image);
  }
}


void MainFrame::LoadImageFromList(int index)
{
  if (_FileNames.size() == 0)
  {
    //The CANVASES should be emptied and refreshed (somehow they don't
    //do it automatically, so we have to make it M-A-N-U-A-L-L-Y

    //P.S. Somehow even with the following code written:
    if(_ImageCanvasUC!=NULL){
      _ImageCanvasUC->SetSize(5, 5);
      _ImageCanvasUC->Destroy();
      _ImageCanvasUC = NULL;
    }
    if(_ImageCanvasFloat!=NULL){
      _ImageCanvasFloat->SetSize(5, 5);
      _ImageCanvasFloat->Destroy();
      _ImageCanvasFloat = NULL;
    }

    _SizerWindow->SetSize(5, 5);
    _SizerWindow->Refresh();
    _SizerWindow->Update();

    //it doesn't work - the CANVAS remains to be shown.
    //So what we do is a little trick:

    /**
    ImageBase image = new ImageBase();
      Camera<unsigned char> camera(image);
      camera.ParseMetaData();
      camera.PrintAppData(info);
    **/

    return;
  }

  _CurrentImage = index;
  const string& fileName = _FileNames[index];

  // load image
  ImageBase image;

  // true disk access 
  if (ImageIO::Load(fileName.c_str(), image)!=0)
  {
    stringstream msg;
    msg << "Error loading " << fileName;
    BIASERR(msg.str());
    SetStatusText(AsciiToWx(msg.str()));
    return;
  }  

  // handle compressed image sizes
  // where display imaeg dimension 
  // differs form orig image dimension.
  // e..g. X3B multicam images (with three images in one file
  // or Pyramid images, depth/color PMD,...
  UncompressImage(image);

  // save meta data for image info dialog
  stringstream info;
  switch (image.GetStorageType())
  {
  case ImageBase::ST_unsignedchar:
    {
      Camera<unsigned char> camera(image);
      camera.ParseMetaData();
      camera.PrintAppData(info);
    } break;

  case ImageBase::ST_float:
    {
      Camera<float> camera(image);
      camera.ParseMetaData();
      camera.PrintAppData(info);
    } break;

  default:
    break;
  }
  _MetaData = info.str();

  // create appropriate image canvas based on storage type
  if (image.GetStorageType() == ImageBase::ST_unsignedchar)
  {
    //check if the previous image was with floats
    bool histo_was_shown_for_float = false;

    // delete float image canvas
    if (_ImageCanvasFloat != NULL)
    {
      //when recreating Histogramm-window, give it the same position as it had
      //before
      _HistoPosition = _ImageCanvasFloat->GetImageCanvas()->GetHistoPos();
      _HistoSize     = _ImageCanvasFloat->GetImageCanvas()->GetHistoSize();
#ifdef BIAS_DEBUG
      cout << "Histogramm Position: " 
           << _HistoPosition.x << " " 
           << _HistoPosition.y << endl;
      cout << "Histogramm Size: " 
           << _HistoSize.GetWidth() << "x" 
           << _HistoSize.GetHeight() << endl;
#endif

      if(_ImageCanvasFloat->GetImageCanvas() != NULL){
        if(_ImageCanvasFloat->GetImageCanvas()->HistogrammIsShown())
          histo_was_shown_for_float = true;
      }

      _MainSizer->Detach(_ImageCanvasFloat);
      _ImageCanvasFloat->Destroy();
      _ImageCanvasFloat = NULL;
    }

    // create uc image canvas
    if (_ImageCanvasUC == NULL)
    {
      _ImageCanvasUC = new ImageCanvas(_SizerWindow, GetStatusBar(), 1);
      // TODO: we need an onmove/onpaint handler to fix flickering. 
      // The ImageCanvas DrawBitamp way has a flicker problem by design JW
      //_ImageCanvasUC->SetClearDrawingContext(false); // avoid flicker
      _MainSizer->Add(_ImageCanvasUC);
    }

    //check if the histogramm should be shown and if YES, then update it
    if(_ShowHistogramm){
      if(!histo_was_shown_for_float){
        //cout << "showing histo" << endl;
        if(_ImageCanvasUC->HistogrammIsShown())
          ShowHistogramm(image);
        else
          _ShowHistogramm = false;
      }
      else{
        cout << "FLOAT TO UC" << endl;
        ShowHistogramm(image);
      }
    }

    // convert image
    Image<unsigned char> ucImage;
    ucImage.StealImage(image);    
    ucImage.SetUID(image.GetUID());

    if (image.GetChannelCount() < _SelectedChannel) {
      BIASERR("Can't show selected Channel, because picture has less channels.")
    }    
    else
    {
      // Shows only the selected channel of image      
      if (_SelectedChannel > 0)
      {
        ImageConvert::GetChannel(ucImage, _SelectedChannel-1);
      }
    }      

    if (_FitCheckBox->IsChecked())
    {
      // get size available for image canvas
      int width, height;
      GetClientSize(&width, &height);

      // compute scale that keeps the aspect ratio and the image is fully visible
      float xscale = (float)width / image.GetWidth();
      float yscale = (float)height / image.GetHeight();
      float scale = xscale < yscale ? xscale : yscale;

      _ImageCanvasUC->SetScaledImageSize((int)(image.GetWidth() * scale), (int)(image.GetHeight() * scale));
      _ImageCanvasUC->SetShowScaled(true);
    }
    else
    {
      _ImageCanvasUC->SetShowScaled(false);
    }

    // show image in canvas
    _ImageCanvasUC->Show(ucImage, fileName);

    // don't show scrollbars if image is maximized
    if (_FitCheckBox->IsChecked())
    {
      _ImageCanvasUC->SetScrollbars(0, 0, 0, 0);
    }
  }
  else
  {
    //check if the previous image was with unsigned char
    bool histo_was_shown_for_uc = false;
    // delete uc image canvas
    if (_ImageCanvasUC != NULL)
    {
      //when recreating Histogramm-window, give it the same position as it had
      //before
      _HistoPosition = _ImageCanvasUC->GetHistoPos();
      _HistoSize     = _ImageCanvasUC->GetHistoSize();
#ifdef BIAS_DEBUG
      cout << "Histogramm Position: " 
           << _HistoPosition.x << " " 
           << _HistoPosition.y << endl;
      cout << "Histogramm Size: " 
           << _HistoSize.GetWidth() << "x" 
           << _HistoSize.GetHeight() << endl;
#endif

      if(_ImageCanvasUC->HistogrammIsShown())
        histo_was_shown_for_uc = true;

      _MainSizer->Detach(_ImageCanvasUC);
      _ImageCanvasUC->Destroy();
      _ImageCanvasUC = NULL;
    }

    // create float image canvas
    if (_ImageCanvasFloat == NULL)
    {
      _ImageCanvasFloat = new ScaledImageCanvas(_SizerWindow, GetStatusBar(), 1);
      _MainSizer->Add(_ImageCanvasFloat);
    }

    //check if the histogramm should be shown and if YES, then update it
    if(_ShowHistogramm){
      if(!histo_was_shown_for_uc){
        //cout << "showing histo" << endl;
        if(_ImageCanvasFloat->GetImageCanvas() != NULL){
          //cout << "canvas exists" << endl;
          if(_ImageCanvasFloat->GetImageCanvas()->HistogrammIsShown())
            ShowHistogramm(image);
          else
            _ShowHistogramm = false;
        }
      }
      else{
        if(_ImageCanvasFloat->GetImageCanvas() != NULL){
          cout << "UC to FLOAT" << endl;
          ShowHistogramm(image);
        }
      }
    }

    /**
    // set appropriate scale and offset range
    switch (image.GetStorageType())
    {
    case ImageBase::ST_unsignedshortint:
      _ImageCanvasFloat->SetScaleRange(1.0/16.0, 1.0);
      _ImageCanvasFloat->SetOffsetRange(-4095.0, 0.0);
      break;

    case ImageBase::ST_char:
      _ImageCanvasFloat->SetScaleRange(0.5, 5.0);
      _ImageCanvasFloat->SetOffsetRange(-128.0, 128.0);
      break;

    case ImageBase::ST_float:
      _ImageCanvasFloat->SetScaleRange(0.5, 50.0);
      _ImageCanvasFloat->SetOffsetRange(-100.0, 100.0);
      break;

    default:
      cout << "Warning: Unknown storage type" << endl;
      // TODO: set default ranges? don't show image at all?
      break;
    }
    **/

    if (image.GetChannelCount() < _SelectedChannel) {
      BIASERR("Can't show selected Channel, because picture has less channels.")
    }    
    else
    {
      // Shows only the selected channel of image      
      if (_SelectedChannel > 0)
      {
        ImageConvert::GetChannel(image, _SelectedChannel-1);
      }
    }      

    if (_FitCheckBox->IsChecked())
    {
      // get size available for image canvas
      int width, height;
      GetClientSize(&width, &height);
      width = max(1, width - 10); // little fix so the image is really fully visible
      height = max(1, height - 40); // larger fix because of sliders in float canvas

      // compute scale that keeps the aspect ratio and the image is fully visible
      float xscale = (float)width / image.GetWidth();
      float yscale = (float)height / image.GetHeight();
      float scale = xscale < yscale ? xscale : yscale;

      _ImageCanvasFloat->GetImageCanvas()->SetScaledImageSize((int)(image.GetWidth() * scale), (int)(image.GetHeight() * scale));
      _ImageCanvasFloat->GetImageCanvas()->SetShowScaled(true);
    }
    else
    {
      _ImageCanvasFloat->GetImageCanvas()->SetShowScaled(false);
    }

    // show image in canvas
    _ImageCanvasFloat->Show(image, fileName);

    // don't show scrollbars if image is maximized
    if (_FitCheckBox->IsChecked())
    {
      _ImageCanvasFloat->GetImageCanvas()->SetScrollbars(0, 0, 0, 0);
    }
  }

  stringstream msg;
  msg << "File '" << fileName << "' (" << image.GetWidth() << "x" << image.GetHeight() << ") loaded";
  SetStatusText(AsciiToWx(msg.str()));

  this->RefitClients();

  // breaks WIN32
  //#ifndef WIN32
  // make the shortcuts work without the need to click the canvas manually (on linux)
  //if (_ImageCanvasUC) 
  //  _ImageCanvasUC->SetFocus();
  //if (_ImageCanvasFloat) 
  //  _ImageCanvasFloat->SetFocus();
  //#endif
}

void MainFrame::LoadImages(const wxArrayString &paths)
{
  // add file names to internal std::string list
  bool ok=true;
  for (int i = 0; i < (int)paths.GetCount(); i++) {
    wxString extUP=wxFileName(paths[i]).GetExt().Upper();
    if (wxFileName(paths[i]).IsDir()) {
      ok=false;
    } else if ((extUP==AsciiToWx("LST")) || (extUP==AsciiToWx("TXT"))) {
      // special handling for .lst and .txt files. 
      // they are no images but contain a list of line separated image 
      //filenames
      vector<string> lstFilenames;
      int result=BIAS::Param::ParseListFile(WxToAscii(paths[i]),
        lstFilenames);
      if (result!=0){
        BIASERR("could not load image list file "<<paths[i].c_str()
          <<" result="<<result<<endl);
      } else {
        // append filenames
        _FileNames.insert(_FileNames.end(), lstFilenames.begin(),
          lstFilenames.end() );
      }
    } else {
      _FileNames.push_back( string(WxToAscii(paths[i])) );
    }
  }

  // load last added file and remember file name and directory
  // do NOT jump to file 0 if adding files subsequently. 
  // we may discuss to to jump to first of cuurently "added" file, 
  // but never jump to first file in list of all files! JW
  if (!_FileNames.empty()) {
    LoadImageFromList(_FileNames.size() - 1);
    _DefaultFile = AsciiToWx(_FileNames[_FileNames.size() - 1]);
    //LoadImageFromList(0);
    //_DefaultFile = AsciiToWx(_FileNames[0]);
  }

  if (!ok) {
    wxMessageBox(wxT("Dropping directories is unsupported. Please drop only "
      "filenames!"),
      wxT("Drag&Drop Error"), wxOK | wxICON_EXCLAMATION);
  }
}

void MainFrame::FitSizetIfTooSmall()
{

  int w=0, h=0;
  int imgW=0, imgH=0;
  if (_ImageCanvasUC!=NULL){
    if (_ImageCanvasUC->GetOrigImage()==NULL)
      return;
    imgW = _ImageCanvasUC->GetOrigImage()->GetWidth();
    imgH = _ImageCanvasUC->GetOrigImage()->GetHeight();
  } else if (_ImageCanvasFloat!=NULL){
    imgW = _ImageCanvasFloat->GetOrigimg().GetWidth();
    imgH = _ImageCanvasFloat->GetOrigimg().GetHeight();
  } else {
    // no valid canvas, nothing to do.
    return;
  }
  // get current frame size, TODO: query client area of canvas instead.
  this->GetClientSize(&w, &h);

  // query display screen dim
  int screenW=0, screenH=0;
  ::wxDisplaySize(&screenW, &screenH);
  if ( (w<imgW) || (h<imgH)) {
    // expand limited by screen size
    // TODO: adpat Client size, to avoid the FitInside...
    // problem: ImageCanves do not handle sizers correctly...
    // this is an approximation working for now: 
    this->SetSize(
      //0,0, /* move win to top-left of screen */
      //-1, -1, /* keep window pos. */
      min(imgW+40, screenW), min(imgH+100, screenH) );
  }
  this->RefitClients();
}


void MainFrame::RefitClients()
{
  int w, h;

  GetClientSize(&w, &h);

  // canvas needs to be refitted on windows after loading (not sure why)
  if (_ImageCanvasUC!=NULL){
    _ImageCanvasUC->SetSize(
      //0,0,
      //-1, -1, 
      w, h);//< keep window pos., do not move the window!
  }
  if (_ImageCanvasFloat!=NULL){
    _ImageCanvasFloat->SetSize(
      //0,0,
      //-1, -1, 
      w, h); //< keep window pos., do not move the window!
  }
  if(_ImageCanvasUC != NULL || _ImageCanvasFloat != NULL){
    _SizerWindow->SetSize(w,h);
  }
}

void MainFrame::OnLoad(wxCommandEvent& /*event*/)
{
  wxFileDialog fileDialog(
    this,                          // parent
    wxT("Choose file(s) to load"), // message
    _DefaultDir,                   // default directory
    _DefaultFile,                  // default file
    wxT("*.*"),                    // wildcard
    wxOPEN | wxMULTIPLE);          // style

  if (fileDialog.ShowModal() == wxID_OK)
  {
    // get file names
    wxArrayString paths;
    fileDialog.GetPaths(paths);
    // load them
    LoadImages(paths);
  } else {
    SetStatusText(wxT("Loading canceled"));
    cout << "Canceled loading image" << endl;
  }

  _DefaultDir = fileDialog.GetDirectory();
}


void MainFrame::AddInfo(const BIAS::ImageBase & image, std::stringstream &info){
  info << "Width:    " << image.GetWidth() << endl;
  info << "Height:   " << image.GetHeight() << endl;
  info << "Depth:    " << image.GetDepth() << endl;
  info << "BitDepth: " << image.GetBitDepth() << endl;
  info << "SizeByte: " << image.GetSizeByte() << " bytes" << endl;
  info << "Pixel Count:   " << image.GetPixelCount() << endl;
  info << "Channels:      " << image.GetChannelCount() << endl;
  info << "ColorModel:    " << image.GetColorModel()  << endl;
  info << "StorageType:   " << image.GetStorageType() << endl;
  info << "IsPlanar:      " << image.IsPlanar() << endl;
  info << "IsInterleaved: " << image.IsInterleaved() << endl;
  info << "UUID:          " << image.GetUID() << endl;
  info << "Version:       " << image.GetVersionNumber() << endl;
}

void MainFrame::OnInfo(wxCommandEvent& /*event*/)
{
  // get currently displayed image
  ImageBase* image = NULL;
  if (_ImageCanvasUC != NULL)
  {
    image = _ImageCanvasUC->GetOrigImage();
  }
  else if (_ImageCanvasFloat != NULL)
  {
    image = &_ImageCanvasFloat->GetOrigimg();
  }
  if (image == NULL)
  {
    wxMessageBox(wxT("No image loaded"), wxT("Image Info"), wxOK | wxICON_ERROR, this);
    return;
  }
  
  // collect information
  stringstream info;
  BIASASSERT(image!=NULL);
  AddInfo(*image, info);

  // some info may already be lost through by Uncompress pre-conversion
  if (infoRaw_.size()>0)
    info << endl<<endl<<"RAW info before uncompression: "<<endl<<infoRaw_<<endl;

  // show information in message box
  wxMessageBox(AsciiToWx(info.str()), wxT("Image Info"), wxOK, this);
}

void MainFrame::OnImageValues(wxCommandEvent& /*event*/)
{
  float dmin=0.0, dmax=0.0;
  // get currently displayed image
  ImageBase* image = NULL;
  if (_ImageCanvasUC != NULL) {
    image = _ImageCanvasUC->GetOrigImage();
    Image<unsigned char> tim(*image);
    unsigned char min, max;
    tim.GetMinMaxPixelValue(min, max);
    dmin = min;
    dmax = max;
  } else if (_ImageCanvasFloat != NULL){
    image = &_ImageCanvasFloat->GetOrigimg();
    Image<float> tim(*image);
    tim.GetMinMaxPixelValue(dmin, dmax);
  }
  if (image == NULL){
    wxMessageBox(wxT("No image loaded"), wxT("Image Values"), 
                 wxOK | wxICON_ERROR, this);
    return;
  }

  // collect information
  stringstream info;
  info << "min pixel value:   " << dmin << endl;
  info << "max pixel value:   " << dmax << endl;
  
  // show information in message box
  wxMessageBox(AsciiToWx(info.str()), wxT("Image Values"), wxOK, this);
}

void MainFrame::OnMetaData(wxCommandEvent& /*event*/)
{
  wxFont font(10, wxMODERN, wxNORMAL, wxNORMAL, false, wxT("Courier"));
  wxDialog* dlg = new wxDialog(this, -1, wxT("Meta Data"), wxDefaultPosition, wxDefaultSize,
    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);
  wxTextCtrl* text = new wxTextCtrl(dlg, -1, wxT(""),
    wxDefaultPosition, wxSize(650, 400),
    wxTE_MULTILINE | wxTE_READONLY | wxTE_DONTWRAP);
#ifdef WIN32
  text->SetFont(font);
#endif
  text->SetDefaultStyle(wxTextAttr(wxNullColour, wxNullColour, font));
  text->AppendText(AsciiToWx(_MetaData));
  text->SetInsertionPoint(0);
  dlg->Fit();
  dlg->ShowModal();
  dlg->Destroy();
}

void MainFrame::OnExit(wxCommandEvent& /*event*/)
{
  Destroy();
}

void MainFrame::OnKeyInfo(wxCommandEvent& /*event*/)
{
  wxString text =
    wxT("Next Image\t[UP]\n")
    wxT("Prev Image\t[UP]\n")
    wxT("Run-Stop\t[Space]");

  wxMessageBox(text, wxT("Keys"),
    wxOK | wxICON_INFORMATION, this);
}

void MainFrame::OnAbout(wxCommandEvent& /*event*/)
{
  stringstream msg;
  string version = "$Revision: 2.133 $";
  version = version.substr(9, version.length()-10);

  msg << "The BiasViewWX Image Viewer\n\n"
    << "http://www.mip.informatik.uni-kiel.de\n\n"
    << "built on " << __DATE__ << " " << __TIME__ << "\n"
    << "from file " << __FILE__ << "\n"
    << "with cvs file version " << version << "\n\n"
    << "GUI using: " << wxVERSION_STRING << "\n"
    << "\n";

  wxMessageBox(AsciiToWx(msg.str()), wxT("About biasviewwx"),
    wxOK | wxICON_INFORMATION, this);
}


void MainFrame::OnRemoveCurrent(wxCommandEvent& /*event*/){
  cout << "Image NR:" << _CurrentImage << " out of " 
       << (int)_FileNames.size() << " images is currently removed" << endl;

  if (_CurrentImage < (int)_FileNames.size()){  
    _FileNames.erase(_FileNames.begin()+_CurrentImage);
  }
  if(_CurrentImage < (int)_FileNames.size()-1){
    _CurrentImage++;
  }
  else
    _CurrentImage=0; 

  LoadImageFromList(_CurrentImage);

  /**
  //if all the images are removed from the list, repaint the screen empty
  if((int)_FileNames.size() == 0){
    _ImageCanvasUC = NULL;
    _ImageCanvasFloat = NULL;

    _SizerWindow->Refresh();
    _SizerWindow->Update();
  }
  **/
}

void MainFrame::OnRemoveAllButCurrent(wxCommandEvent& /*event*/){
  cout << "Image NR:" << _CurrentImage << " out of " 
       << (int)_FileNames.size() << " images is currently loaded" 
       << " before erasing all but current" << endl;
  if (_FileNames.size()<1){
    SetStatusText(_T("no images"));
    return; // nothing to to
  }
  BIASASSERT( (int)_FileNames.size() > _CurrentImage);
  // make current the first and only image in list
  _FileNames[0] = _FileNames[_CurrentImage];
  _CurrentImage=0;
  _FileNames.resize(1);
  // update display
  LoadImageFromList(_CurrentImage);
  cout << "Image NR:" << _CurrentImage << " out of " 
       << (int)_FileNames.size() << " images is currently loaded" 
       << " after erasing all but current" << endl;
}


void MainFrame::OnSaveList(wxCommandEvent& /*event*/)
{
  wxString filename = wxFileSelector(_T("Choose file to save the image list to.")
				     ,_T("") // default path
				     ,_T("out_images.lst") //def. filename
				     ,_T(".lst") // def. extension
				     ,_T("LST files (*.lst)|*.lst|TXT files (*.txt)|*.txt|All files (*.*)|*.*") // wildcard
				     ,wxSAVE|wxOVERWRITE_PROMPT // flags
				     );
  if ( !filename.empty() ){
    ofstream f( WxToAscii(filename) ); // ios::out overwrite, not append
      if (!f.good()){
	stringstream msg; msg<<"could nor open "<<filename.c_str()<<" for writing";
	SetStatusText( AsciiToWx( msg.str() ));
	BIASERR(msg.str());
	return;
      }
    vector<string>::const_iterator it = _FileNames.begin();
    int n=0;
    for ( ;it<_FileNames.end(); it++){
      f<<*it<<endl;
      n++;
    }
    f.close();
    stringstream msg; msg<<"wrote "<<filename.c_str()<<" with "<<n<<" filenames";
    SetStatusText( AsciiToWx(msg.str() ));
  }
}

void MainFrame::OnNext(wxCommandEvent& /*event*/)
{
  _CurrentImage++;
  if (_CurrentImage >= (int)_FileNames.size())
  {
    _CurrentImage = 0;
  }
  LoadImageFromList(_CurrentImage);
}

void MainFrame::OnPrev(wxCommandEvent& /*event*/)
{
  _CurrentImage--;
  if (_CurrentImage < 0)
  {
    _CurrentImage = _FileNames.size() - 1;
  }
  LoadImageFromList(_CurrentImage);
}

void MainFrame::OnRunStop(wxCommandEvent& /*event*/)
{
  cout<<"DBG: Runstop"<<endl;

  if (_Timer.IsRunning())
  {
    _Timer.Stop();
    _RunStopButton->SetLabel(wxT(" Run [Space] "));
  }
  else
  {
    _Timer.Start(_DelaySpin->GetValue());
    _RunStopButton->SetLabel(wxT(" Stop [Space] "));
  }
}

void MainFrame::OnHistogramm(wxCommandEvent& /*event*/)
{
  if (_FileNames.size() > 0){
    const string& fileName = _FileNames[_CurrentImage];
    
    // load image
    ImageBase image;
    if (ImageIO::Load(fileName.c_str(), image)!=0){
      stringstream msg;
      msg << "Error loading " << fileName;
      BIASERR(msg.str());
      SetStatusText(AsciiToWx(msg.str()));
    } 
    
    if(!_ShowHistogramm)
      _ShowHistogramm = true;
    
    ShowHistogramm(image);
  }
  else
    cout << "No Images Loaded (No Image = No Histogramm)" << endl;
}

void MainFrame::ShowHistogramm(ImageBase& image){
  if (_FileNames.size() > 0){
    //Check if float or UC
    switch (image.GetStorageType()){
    case ImageBase::ST_unsignedchar:
      if(_ImageCanvasUC != NULL){
        _ImageCanvasUC->ShowHistogramm(image, _HistoPosition, _HistoSize);
      }
      break;
    case ImageBase::ST_float:
      if(_ImageCanvasFloat != NULL &&
         _ImageCanvasFloat->GetImageCanvas() != NULL){
        _ImageCanvasFloat->GetImageCanvas()->ShowHistogramm(image,
                                                            _HistoPosition,
                                                            _HistoSize);
      }
      break;
    default:
      break;
    }
    this->SetFocus();
  }
  else
    cout << "No Images Loaded (No Image = No Histogramm)" << endl;
}

void MainFrame::OnDelay(wxSpinEvent& /*event*/)
{
  if (_Timer.IsRunning())
  {
    _Timer.Start(_DelaySpin->GetValue());
  }
}

void MainFrame::OnFit(wxCommandEvent& /*event*/)
{
  LoadImageFromList(_CurrentImage);
}

void MainFrame::OnTimer(wxTimerEvent& /*event*/)
{
  if (!_BlockAnimation)
  {
    // block animation until at least one idle event occurs,
    // so the viewer doesn't get stuck if delay is too low.
    _BlockAnimation = true;
    wxCommandEvent dummyEvent;

    if (_ReverseCheckBox->GetValue())
      OnPrev(dummyEvent);
    else
      OnNext(dummyEvent);
  }
}

void MainFrame::OnIdle(wxIdleEvent& /*event*/)
{
  _BlockAnimation = false;
}

void MainFrame::OnSize(wxSizeEvent& event)
{
  if (_FitCheckBox != NULL)
  {
    if (_FitCheckBox->IsChecked())
    {
      LoadImageFromList(_CurrentImage);
    }
  }
  this->RefitClients();
  event.Skip();
}


void MainFrame::OnChangeChannel_1(wxCommandEvent& event)
{
  _SelectedChannel = 1;
  cout<<_SelectedChannel<<". Channel (red?) selected."<<endl;
  LoadImageFromList(_CurrentImage);
}

void MainFrame::OnChangeChannel_2(wxCommandEvent& event)
{
  _SelectedChannel = 2;
  cout<<_SelectedChannel<<". channel (green?) selected."<<endl;
  LoadImageFromList(_CurrentImage);  
}

void MainFrame::OnChangeChannel_3(wxCommandEvent& event)
{
  _SelectedChannel = 3;
  cout<<_SelectedChannel<<". channel (blue?) selected."<<endl;
  LoadImageFromList(_CurrentImage);  
}


void MainFrame::OnChangeChannel_4(wxCommandEvent& event)
{
  _SelectedChannel = 4;
  cout<<_SelectedChannel<<". channel selected."<<endl;
  LoadImageFromList(_CurrentImage);  
}

void MainFrame::OnChangeChannel_5(wxCommandEvent& event)
{
  _SelectedChannel = 5;
  cout<<_SelectedChannel<<". channel selected."<<endl;
  LoadImageFromList(_CurrentImage);  
}

void MainFrame::OnChangeChannel_6(wxCommandEvent& event)
{
  _SelectedChannel = 6;
  cout<<_SelectedChannel<<". channel selected."<<endl;
  LoadImageFromList(_CurrentImage);  
}

void MainFrame::OnChangeChannel_7(wxCommandEvent& event)
{
  _SelectedChannel = 7;
  cout<<_SelectedChannel<<". channel selected."<<endl;
  LoadImageFromList(_CurrentImage);  
}

void MainFrame::OnChangeChannel_8(wxCommandEvent& event)
{
  _SelectedChannel = 8;
  cout<<_SelectedChannel<<". channel selected."<<endl;
  LoadImageFromList(_CurrentImage);  
}


void MainFrame::OnChangeChannel_ALL(wxCommandEvent& event)
{
  cout<<"ALL Channels are selected."<<endl;
  _SelectedChannel = 0;
  LoadImageFromList(_CurrentImage);  
}

/*
===============================================================================
DragAndDropTarget
===============================================================================
*/

/** @brief drag and drop files onto application.
drag a file from MS explorer or kde file manager directly onto an application
(instaed of using a load from filename function.) 
@author Jan Woetzel 03/2005 */
class DragAndDropTarget : public wxFileDropTarget
{
public:
  DragAndDropTarget(MainFrame* frame);

  virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);
protected:
  MainFrame* _Frame;
};

DragAndDropTarget::DragAndDropTarget(MainFrame* frame)
{
  _Frame = frame;
}

bool DragAndDropTarget::OnDropFiles(wxCoord /*x*/, wxCoord /*y*/, const wxArrayString& filenames)
{
  BIASASSERT( _Frame != NULL);
  wxArrayString sortedFilenames(filenames);
  // sort alphabetically because file seq is not preserved "as expected"
  // (at least on Windows XP Explorer)
  sortedFilenames.Sort();
  _Frame->LoadImages(sortedFilenames);
  _Frame->FitSizetIfTooSmall();
  _Frame->RefitClients();

  return true;
}

/*
===============================================================================
BiasViewApp
===============================================================================
*/

/** @class biasviewwx
@ingroup g_tools
@brief tool for displaying images using wxWidgets GUI.
@author many - including woelk,evers,jw,... */
class BiasViewApp : public wxApp
{
public:
  virtual bool OnInit();
};

bool BiasViewApp::OnInit()
{
  // utility: create a Redirection console for cout, cerr etc.
  ConsoleRedirectIO();

  MainFrame* frame =
    new MainFrame(wxT("BiasViewWX"), wxPoint(50, 50), FRAME_DEFAULT_SIZE);

  frame->Move(0,0); // move once on program start to upper left of screen

  // append file list given on command line
  {
    wxArrayString cmdFiles;
    for (int i = 1; i < argc; i++){
      cmdFiles.Add(wxString(argv[i]));
    }
    frame->LoadImages(cmdFiles);
  }

  // adapt Frame size on first loaded image
  frame->FitSizetIfTooSmall();

  // create drag and drop target
  DragAndDropTarget* dndTarget = new DragAndDropTarget(frame);
  frame->SetDropTarget(dndTarget);

  // show frame
  frame->Show(TRUE);
  SetTopWindow(frame);

  return TRUE;
}

IMPLEMENT_APP(BiasViewApp)
