#include <Utils/TriangleMesh.hh>
#include <Utils/ThreeDOut.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <Image/HomographyMapping.hh>
#include <iostream>

using namespace std;
using namespace BIAS;

void usage(char* name) 
{
  cout<<"usage:\n";
  cout<<name<<" <depthmap> <texture> [outputFile]\n";
}

int main(int argc, char* argv[]) 
{
  if(argc<2) {
    usage(argv[0]);
    return -1;
  }

  ImageBase depthMap;
  if(ImageIO::Load(argv[1], depthMap)!=0) {
    cout<<"depth map could not be loaded!\n";
    usage(argv[0]);
    return -1;
  }

  Image<float> source, enlarged;
  ImageConvert::ConvertST(depthMap, source, ImageBase::ST_float);
    
  //Unhardcode Scale Factor
  HomographyMapping<float, float> HMapper;
  HMatrix H(MatrixIdentity);
  H[0][0]=H[1][1]=0.5;
  HMapper.SetHomography(H);
  enlarged.Init(source.GetWidth()*2,source.GetHeight()*2,1 );
  HMapper.Map(source, enlarged);
  Image<unsigned char> texture;
  if(ImageIO::Load(argv[2], texture)!=0) {
    cout<<"texture could not be loaded!\n";
    usage(argv[0]);
    return -1;
  }

  cout<<"meshing..."; cout.flush();
  TriangleMesh mesh;
  
  mesh.GenerateDenseMesh(enlarged, texture);  
  cout<<"finished\n";

  ThreeDOut vrmlOut;
  vrmlOut.AddTriangleMesh(mesh, "mesh_from_"+string(argv[1]), 
                          argv[2], false);

  cout<<"Writing VRML to "; 
  if(argc>3) {
    cout<<argv[3]<<"... ";cout.flush();
    vrmlOut.VRMLOut(argv[3]);
  } 
  else {
    cout<<"DenseTriangleMesh.wrl... ";cout.flush();
    vrmlOut.VRMLOut("DenseTriangleMesh.wrl");
  }
  cout<<"finished\n";
  
  return 0;
}
