/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __GenVisiblePoints_hh__
#define __GenVisiblePoints_hh__

#include <bias_config.h>
#include <Base/Common/BIASpragmaStart.hh>
  
#include <vector>
#include <Base/Math/Random.hh>
#include <Geometry/ProjectionParametersBase.hh>

namespace BIAS {
  
  /** @class GenVisiblePoints
      @ingroup g_utils
      @brief Class for generating 3D points visible by a set of cameras.
      @author jkollmann */
  class BIASUtils_EXPORT GenVisiblePoints
  {
  public:

    /** Generates 3D points visible by the given cameras.
        - pp:
          Projection parameters defining the cameras.
        - numVisiblePointsPerCamera:
          Minimum number of points visible by each camera.
        - numTries:
          Number of tries to find a random point visible by a camera
          but invisible by all previous cameras. Choosing a high number
          will increase the probability that exactly
          'numVisiblePointsPerCamera' points are visible by each camera.
        - minDistance, maxDistance:
          Minimum and maximum distance of generated points to the camera
          center.
        - points:
          List of generated 3D points. */    
    void GeneratePoints(const std::vector<ProjectionParametersBase*>& pp,
                        const int numVisiblePointsPerCamera,
                        const int numTries,
                        const double minDistance,
                        const double maxDistance,
                        std::vector<HomgPoint3D>& points);

  private:

    /** Computes a single random point visible by a camera. */
    void GenerateVisiblePoint(const ProjectionParametersBase& projection,
                              const double minDistance,
                              const double maxDistance,
                              HomgPoint3D& point);

    /** Returns true iff the given point is visible by the given camera. */
    bool IsPointVisible(const ProjectionParametersBase& pp,
                        const HomgPoint3D& point);

    Random _rand;
  };

} // namespace

#include <Base/Common/BIASpragmaEnd.hh>
  
#endif // __GenVisiblePoints_hh__
