/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

                         //#include <iostream>
#include <Utils/Param.hh>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>
#include <Geometry/Projection.hh>


namespace BIAS{
  /** Class grouping together some IO code often used in (small) application,
   * like loading images, parsing their meta data....
   * \author bartczak 01/2008
   */
  class IOUtils {
  public:

    template<class dataType>
    static bool LoadCamera(std::string* fileName, 
                           BIAS::Camera<dataType>& cam) 
    {
      if(ImageIO::Load(*fileName, cam)!=0) {
        BIASERR("could not load camera: "<<*fileName);
        return false;
      }    
      return true;
    }


    template<class dataType>
    static bool LoadCamera(std::string* fileName, 
                           BIAS::Camera<dataType>& cam, 
                           BIAS::Projection& proj) 
    {      
      if(!LoadCamera<dataType>(fileName, cam))
        return false;
        
      if(cam.ParseMetaData()<0) {
        BIASERR("could not parse metadata from file: "<<*fileName);
        return false;
      }
      proj = cam.GetProj();
      return true;
    }

    template<class dataType>
    static bool SaveCamera(const std::string& fileName, 
                           const BIAS::Camera<dataType>& cam) 
    {
      if(ImageIO::Write(fileName, cam)!=0) {
        BIASERR("could not save camera: "<<fileName);
        return false;
      }    
      return true;
    }
    
    template<class dataType>
    static bool SaveCamera(const std::string& fileName, 
                           BIAS::Camera<dataType>& cam,
                           const BIAS::Projection& proj) 
    {
      cam.SetProj(proj);
      cam.UpdateMetaData();
      if(!SaveCamera<dataType>(fileName, cam))
        return false;

      return true;      
    }

    static bool ParseCommandLineEvalHelp(Param& params, int argc, char* argv[]){
      int miscGrpID = params.GetFreeGroupID();
      bool* help = params.AddParamBool("help", "", false, 'h', miscGrpID);
      params.SetGroupName(miscGrpID, "misc");

      params.ParseCommandLine(argc, argv);
      if(*help) {
        params.Usage();
        return false;
      }
      return true;
    }
  }; 

} //end of namespace
