#include <Image/Camera.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>
#include <VideoSource/VideoSource_DCAM.hh>
//#include <unistd.h>
//#include <getopt.h>

using namespace BIAS;
using namespace std;

#include <Base/Debug/TimeMeasure.hh>

#define WIDTH 1280
#define HEIGHT 960
#define ULX 0
#define ULY 0
#define PACKAGESIZE 3840


int main(int argc, char *argv[])
{
  int res = -1;
  
  VideoSource_DCAM *cam;
  Camera<unsigned char> grabimg;
  // prepare camera
  cam = new VideoSource_DCAM;

  cam[0].SetDebugLevel(D_DCAM |D_DCAM_F7);
  //  cam[1].SetDebugLevel(D_DCAM |D_DCAM_F7);

  cam->UsePort(0);
  cam->SetSize(WIDTH, HEIGHT, 3);
  cam->SetLeftTop(ULX, ULY);
  cam->SetFormat7(3, PACKAGESIZE, 3);
  cam->SetColorModel(ImageBase::CM_PGR_XB3_F7M3_GBRG);
  //  cam[1].UsePort(1);

  res = cam->OpenDevice() ;
  if (res <0) {
    cerr<<" Can not open camera on bus "<<res<<endl;
    exit(1);
  }
  cam->PreGrab();
  //   cam[i].SetDebugLevel(0);
  cam->InitImage(grabimg);

  cam->GrabSingle(grabimg);

  unsigned char* grabedPixels = grabimg.GetImageData();
  Camera<unsigned char> triclopsImage[3];
  unsigned char* sinkPixels;
  
  for(unsigned int j=0; j<3; j++) {
    triclopsImage[j].Init(WIDTH, HEIGHT, 1);
    triclopsImage[j].SetColorModel(ImageBase::CM_Bayer_GBRG);
    sinkPixels = triclopsImage[j].GetImageData();

    for(unsigned int h=0; h<HEIGHT; h++) {
      for(unsigned int w=0; w<WIDTH; w++) {
        sinkPixels[h*WIDTH + w] = grabedPixels[h*WIDTH*3 + w*3 + j]; 
      }
    }
  }

  //  ImageIO::Write("cam0",grabimg);
  
  //the names for the images are chosen correctly if the camera is mounted
  //so the screw thread is facing downwords (towards earth's center of gravity)
  ImageIO::Write("xb3_right",triclopsImage[0]);
  ImageIO::Write("xb3_center",triclopsImage[1]);
  ImageIO::Write("xb3_left",triclopsImage[2]);

  // ImageIO::Write("cam1",grabimg[1]);

  cout <<"Write finished, closing cameras"<<endl;

  cam->SetDebugLevel(D_DCAM );
  cam->PostGrab();
  cam->CloseDevice();


  return 0;
}
