#include <Image/Camera.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>
#include <VideoSource/VideoSource_DV.hh>
#include <Gui/GuiGTK.hh>
#include <unistd.h>
#include <getopt.h>

using namespace BIAS;
using namespace std;

#define StorageType unsigned char

#include <Base/Debug/TimeMeasure.hh>

void mainloop(VideoSource_DV *cam)
{
  GuiGTK CamWin;
  bool done = false;
  char c;
  char title[100];
  
  struct timeval now;
  float fps;
  int i,savedframes, savecounter=0;
  
  long start=0, end;
  bool color, pause,  autosave=false;
  ostringstream os;
  
  TimeMeasure t1,t2;

  Camera<unsigned char> pic,pic2;

  sprintf(title,"MyWindow");
  CamWin.SetTitle(title);
  cam->InitImage(pic);
  pic2.Init(720, pic.GetHeight(),3);
  
  pause = false;
  color = true;

  i = 1;
  savedframes = 0;

  
  bool save;
  //pic2.SetDebugLevel(D_CONVERT);
  
  while (!done){ 
    save = false;
    gettimeofday(&now, NULL);
    end = start;
    start =  (now.tv_sec % 1000) * 1000 + (now.tv_usec / 1000);
    fps = 1.0 / (float)(start-end) * 1000.0 ;

    c = CamWin.CheckForKeyEvent();
    switch (c) {
    case 'q': 
      done = true;
      break;
    case 's':
      save = true;
      break;
    case 'S':
      autosave=!autosave;
      savecounter=0;
      if (autosave) cerr << "\nstarted saving sequence at frame "<<i<<endl;
      else cerr << "\nstopped saving sequence at frame "<<i<<endl;
      break;
    case ' ': 
      pause = !pause;	  
      break;
    case 'c': 
      color = !color;
      if (color) cerr << "\nshowing color image" << endl;
      else cerr << "\nshowing grey image" << endl;
      break;
    default:
      if (c!=-1) cerr <<"unknown key "<<c<<endl;
      break;
    }

            
    if (!pause) {
//       cout <<"starting grab"<<endl;
      cam->GrabSingle(pic);
//       cout <<"got image"<<endl;

      ImageConvert::ToRGB(pic, pic2);
    
      //if (i==10)
      //  pic2.ExportImage("Cam.pnm",MIP_pnm);
      if (save) {
        ImageIO::Write("image",pic2);
        //  	    ImageIO::ExportImage("image.pnm",pic2,ImageIO::FF_ppm);
        save = false;
      }
      if (autosave){
        os.str("");
        os << "frame" << setfill('0') << setw(5) << savecounter;
        if (ImageIO::ExportImage(os.str(),pic2, ImageIO::FF_pgm)!=0){
          BIASERR("error saving frame "<<i<<" at file "<<os.str()<<".pgm");
        }
        savecounter++;
      }

      CamWin.ShowImage(pic2);

      cout <<"Frame: "<<i<<"  Showing  fps: "<<fps<<"\r";
      cout.flush();
	  
      i++;
    } // if (!pause) 
  }  
}







int main(int argc, char *argv[])
{
  int res;


  VideoSource_DV *cam;

  // prepare camera
  cam = new VideoSource_DV;

  cam->SetDebugLevel(D_DV);
  
//   cam->SetDebugLevel(D_DCAM);
  res = cam->OpenDevice() ;
  cout <<"Openresult: "<<res<<endl;
  
  if (res <0) {
    cerr<<" Can not open camera"<<endl;
    exit(1);
  }

  //cam->ShowCapabilities();
  cam->PreGrab();

  cout <<" s  : save an image \n";
  cout <<" S  : save sequence\n";
  cout <<"SPC : Stop/Resume\n";
  cout <<" e  : toggle between slow and fast conversion\n";
  cout <<" q  : Quit      \n";


  mainloop(cam );
  
  cam->PostGrab();

  cam->CloseDevice();
  delete cam;
  
  return 0;
}
