#ifndef __MIPVIDEOSOURCE_DSHOW_HH__
#define __MIPVIDEOSOURCE_DSHOW_HH__

#include <bias_config.h>


#include <atlbase.h>
#include <windows.h>
#include <dshow.h>
#include <stdio.h>
#include <VideoSource/VideoSource_Base.hh>
#include <qedit.h>
#include <comutil.h>
#include <strmif.h>
#include <Image/Camera.hh>
 
namespace BIAS{

/** 
    \class VideoSource_DSHOW
    \ingroup VideoSource
    \author ischiller
    \brief This class extends VideoSource for the use of DirectShow devices
*/


#if !defined __MYSAMPLE_GRABBER_CB_H
#define __MYSAMPLE_GRABBER_CB_H

class MySampleGrabberCB : public ISampleGrabberCB {

protected:
 
	// The media type we are processing
	AM_MEDIA_TYPE		mediaType;

public:

  BIAS::Camera<unsigned char> camera_;
	
  // donstructor
  MySampleGrabberCB(int width, int height, int depth);
	// destructor
	~MySampleGrabberCB();

    // This is the callback method called every time 
  STDMETHODIMP SampleCB(double SampleTime, IMediaSample *pSample);
	// the other TO IMPLEMENT method
	STDMETHODIMP BufferCB(double SampleTime, BYTE *pBuffer, long BufferLen);

	STDMETHODIMP QueryInterface(REFIID riid, void ** ppv);
	STDMETHODIMP_(ULONG) AddRef() { return 2; }
  STDMETHODIMP_(ULONG) Release() { return 1; }

};


#endif

  class VideoSource_DSHOW : public VideoSource{
  
 
  public: 
  VideoSource_DSHOW(bool bShowDialogs = false);
  VideoSource_DSHOW(int width, int height,int depth,bool bShowDialogs = false);

  ~VideoSource_DSHOW();

  int OpenDevice(unsigned int width, unsigned int height, unsigned int channels,
    const char * device,BIAS::ImageBase::EColorModel model = BIAS::ImageBase::CM_RGB);
  int OpenDevice(const char * device);
  int CloseDevice();

  int GrabSingle(BIAS::Camera <unsigned char>  &image);
  int InitImage(BIAS::ImageBase &Image);
  
 
  int PreGrab();
  int PostGrab();

  inline void SetCaptureDeviceName(std::string sName){
    sCaptureDeviceName_ = sName;
  ;};

  inline void SetSizeAndColorModel(int width, int height, 
    int bytesperpixel=1, BIAS::ImageBase::EColorModel model = BIAS::ImageBase::CM_invalid){
     SetSize( width, height, bytesperpixel);
     SetColorModel(model);
     BIASColorModel_= model;
    };

  bool DisplayCameraProperties();

  void DisplaySourcePinProperties();
  
  bool DisplayFilterProperties( CComPtr<IBaseFilter> pFilter);
 
protected:
  MySampleGrabberCB*   FGCB;
	long	            dDsBufferSize_;
	long *	          pDsBuffer_;
  unsigned char *   pImageBuffer_;
	int		dWidth_;
	int		dHeight_;
	int		dDepth_;
  bool  bShowDialogs_;

  BIAS::Camera<unsigned char> camera_;

  std::string sCaptureDeviceName_;

	HRESULT InitVideoCapture_();
	HRESULT GetInterfaces_(void);	
	HRESULT FindCaptureDevice_(IBaseFilter ** ppSrcFilter);
  HRESULT EnumFilters_(IFilterGraph *pGraph);

  void DetermineColorModel_();
  void DetermineBIASColorModel_();

	void DeleteMediaType_(AM_MEDIA_TYPE *pmt);

  bool DisplayPinProperties_( CComPtr<IPin> pSrcPin );
  
	HRESULT SaveGraphFile_(IGraphBuilder *pGraph, WCHAR *wszPath);

  BIAS::ImageBase::EColorModel BIASColorModel_;
  GUID MediaSubTypeColorModel_;

  CComPtr< IMediaControl >  			  pMediaControl_;
	CComPtr< IMediaEventEx >				  pMediaEvent_;

  CComPtr< IPin >						        pOutPin_;
  CComPtr< IPin >						        pSampleOutPin_;
	CComPtr< IPin >						        pInPin_;
  CComPtr< ISampleGrabber >         pGrabber_;
  CComPtr< IBaseFilter >		        pNullRenderer_;
	CComPtr< IGraphBuilder >          pGraphBuilder_;
	CComPtr< ICaptureGraphBuilder2 >  pCaptureGraphBuilder2_;    
  CComPtr< IBaseFilter >            pSrcFilter_;  
  }; //end class
}//end namespace mip
#endif
