#ifdef WIN32
# pragma warning (disable: 4251) // STL MAP DLL warning noise (JW)
#endif

#include "VideoSource_Stereo.hh"

#include <Image/Camera.hh>
#include <string>

using namespace BIAS;
using namespace std;

BIAS::VideoSource_Stereo::
VideoSource_Stereo(VideoSource *left, VideoSource *right)
  : VideoSource(), Left_(left), Right_(right)
{}

BIAS::VideoSource_Stereo::
~VideoSource_Stereo()
{
  if (Left_){
    delete Left_;
    Left_ = NULL;
  }
  if (Right_){
    delete Right_;
    Right_ = NULL;
  }
}

int BIAS::VideoSource_Stereo::
OpenDevice(const char *name)
{
  string mname = name;
  string::size_type pos = mname.find(':', 0); 
  if (pos == string::npos){
    BIASERR("invalid name \""<<name<<"\"");
    return -1;
  }

  string lname = mname.substr(0, pos);
  string rname = mname.substr(pos+1, mname.size()-pos-1);
  cout << "lname = \""<<lname<<"\"\trname = \""<<rname<<"\"\n";
  int resl = Left_->OpenDevice(lname.c_str());
  int resr = Right_->OpenDevice(rname.c_str());
  return (resl==0 && resr==0)?0:-1;
}


int BIAS::VideoSource_Stereo::
OpenDevice()
{
  BIASERR("use OpenDevice(char *)");
  BIASABORT;
  return -1;
}

int BIAS::VideoSource_Stereo::
PreGrab()
{
  int resl = Left_->PreGrab();
  int resr = Right_->PreGrab();
  return (resl==0 && resr==0)?0:-1;
}

int BIAS::VideoSource_Stereo::
GrabSingle(BIAS::Camera<unsigned char> &image)
{
  int resl = Left_->GrabSingle(LeftIm_);
  int resr = Right_->GrabSingle(RightIm_);
  image = LeftIm_;
  return (resl==0 && resr==0)?0:-1;
}

int BIAS::VideoSource_Stereo::
PostGrab()
{
  int resl = Left_->PostGrab();
  int resr = Right_->PostGrab();
  return (resl==0 && resr==0)?0:-1;
}

void BIAS::VideoSource_Stereo::
SetSize(int w, int h, int bytesperpixel)
{
  Left_->SetSize(w, h, bytesperpixel);
  Right_->SetSize(w, h, bytesperpixel);
}

int BIAS::VideoSource_Stereo::
CloseDevice()
{
  int resl = Left_->CloseDevice();
  int resr = Right_->CloseDevice();
  return (resl==0 && resr==0)?0:-1;
}


int BIAS::VideoSource_Stereo::
InitImage(BIAS::ImageBase & img)
{ 
  Left_->InitImage(LeftIm_);
  Right_->InitImage(RightIm_);
  return Left_->InitImage(img); 
}

BIAS::VideoSource_Stereo::
VideoSource_Stereo()
{
  BIASABORT;
}

BIAS::VideoSource_Stereo::
VideoSource_Stereo(const VideoSource_Stereo & /*source*/)
{
  BIASABORT;
}


VideoSource_Stereo& BIAS::VideoSource_Stereo::
operator=(const VideoSource_Stereo & /*source*/)
{
  BIASABORT;
  return *this;
}

